

try:
    import tkinter as tk
    from tkinter.filedialog import askopenfilename

    class Window(tk.Tk):
        def __init__(self):
            tk.Tk.__init__(self)
            self.title("scicast")
            self.path = tk.StringVar()
            self.cell_path = tk.StringVar()
            self.gene_path = tk.StringVar()
            self.gene_label_path = tk.StringVar()
            self.exclude_gene_path = tk.StringVar()
            self.asset = tk.StringVar()
            self.gene_number = tk.IntVar(value=200)
            self.depth_number = tk.IntVar(value=20)
            self.kmeans_cluster_range = tk.StringVar(value='2,4')
            self.color_cells = tk.StringVar()
            self.color_genes = tk.StringVar()
            self.test_clust_stability = tk.IntVar(value=0)
            self.genes_corr = tk.StringVar()
            self.annotate_gene_subset = tk.StringVar()

            #type or choose gene matrix file
            dir_label = tk.Label(self, text="Browse or type path to gene cell matrix file:")
            path_entry = tk.Entry(self, textvariable=self.path, width=40)
            browse_button = tk.Button(self, text="Browse for gene cell matrix file", command=self.browse)

            #type or choose cell group file
            cell_label = tk.Label(self, text="Browse or type path to cell group file:")
            cell_path_entry = tk.Entry(self, textvariable=self.cell_path, width=40)
            cell_browse_button = tk.Button(self, text="Browse for cell group file", command=self.browse_cellp)

            #type or choose gene group file
            gene_label = tk.Label(self, text="Browse or type path to gene group file:")
            gene_path_entry = tk.Entry(self, textvariable=self.gene_path, width=40)
            gene_browse_button = tk.Button(self, text="Browse for gene group file", command=self.browse_genep)

            #type or choose file of genes to exclude from all analysis
            exclude_gene_label = tk.Label(self, text="Browse or type path to file with genes to exclude from analysis (i.e cell cycle):")
            exclude_gene_path_entry = tk.Entry(self, textvariable=self.exclude_gene_path, width=40)
            exclude_gene_browse_button = tk.Button(self, text="Browse for exclude genes file", command=self.browse_excludeg)

            #type or choose file of genes which will be labeled in gene PCA plot
            annotate_gene_subset_label = tk.Label(self, text="Annotate only these genes in gene PCA:")
            annotate_gene_subset_path_entry = tk.Entry(self, textvariable=self.annotate_gene_subset, width=40)
            annotate_gene_subset_browse_button = tk.Button(self, text="Browse gene annotation file", command=self.browse_annotateg)

            #define file extensions
            self.file_opt = options = {}
            options['defaultextension'] = '.txt'
            options['filetypes'] = [('all files', '.*'), ('text files', '.txt'),('csv files', '.csv')]

            #setup metric menu options
            self.metric_menu_var = tk.StringVar()
            self.metric_menu_var.set("seuclidean")
            metric_menu_label = tk.Label(self, text="Choose Metric:")
            metric_option_menu = tk.OptionMenu(self, self.metric_menu_var, 'braycurtis', 'canberra', 'chebyshev', 'cityblock', 'correlation', 'cosine', 'dice', 'euclidean', 'hamming', 'jaccard', 'kulsinski', 'mahalanobis', 'matching', 'minkowski', 'rogerstanimoto', 'russellrao', 'sokalmichener', 'sokalsneath', 'sqeuclidean', 'yule')

            #setup method option menu
            self.method_menu_var = tk.StringVar()
            self.method_menu_var.set("ward")
            method_menu_label = tk.Label(self, text="Choose Method:")
            method_option_menu = tk.OptionMenu(self, self.method_menu_var, 'single', 'complete', 'average', 'weighted', 'centroid', 'median')

            #setup qgraph option menu
            self.qgraph_menu_var = tk.StringVar()
            self.qgraph_menu_var.set("none")
            qgraph_menu_label = tk.Label(self, text="Choose which qgraph networks to generate:")
            qgraph_option_menu = tk.OptionMenu(self, self.qgraph_menu_var, 'gene','cell','both')

            #setup z-direction option menu
            self.zdir_menu_var = tk.IntVar()
            self.zdir_menu_var.set(0)
            zdir_menu_label = tk.Label(self, text="Choose z:")
            zdir_option_menu = tk.OptionMenu(self, self.zdir_menu_var, 1,'None')

            self.flags = ["Don't Run Heatmaps","Don't Run Correlation", "Verbose", "Test Significance by Groups (User Defined)", "Test Significance by Unbiased Clusters", "Exclude Cells Not in User Cell Groups", "Add Ellipse", "Add Cell Names to PCA", "Display Only Unique Signifcant Genes", "Run Significance Test for kmeans clusters", "Input Matrix is already log2"]
            self.variables = []



            asset_label = tk.Label(self, text="Output File Name:")
            asset_entry = tk.Entry(self, textvariable=self.asset, width=40)

            gene_number_label = tk.Label(self, text="Number of genes to include")
            gene_number_entry = tk.Entry(self, textvariable=self.gene_number, width=10)

            kmeans_range_label = tk.Label(self, text="Range of cluster for kmeans (inclusive):")
            kmeans_range_entry = tk.Entry(self, textvariable=self.kmeans_cluster_range, width=10)

            depth_number_label = tk.Label(self, text="Depth at which subclustering will stop")
            depth_number_entry = tk.Entry(self, textvariable=self.depth_number, width=10)

            color_cells_label = tk.Label(self, text="Provide specific colors and markers for each cell group.")
            color_cells_entry = tk.Entry(self, textvariable=self.color_cells, width=20)

            color_genes_label = tk.Label(self, text="Provide specific colors and markers for each gene group.")
            color_genes_entry = tk.Entry(self, textvariable=self.color_cells, width=20)

            test_clust_stability_label = tk.Label(self, text="Number of iterations to test cluster stability with varying gene numbers.")
            test_clust_stability_entry = tk.Entry(self, textvariable=self.test_clust_stability, width=10)

            genes_corr_label = tk.Label(self, text="Comma seperated list of genes to add to correlation search.")
            genes_corr_entry = tk.Entry(self, textvariable=self.genes_corr, width=20)


            create_button = tk.Button(self, text="Run scicast", command=self.genAsset)

            dir_label.grid(row=1, column=1, columnspan=2, sticky='w')
            path_entry.grid(row=2, column=1, columnspan=2, sticky='w')
            browse_button.grid(row=3, column=1, columnspan=2, sticky='w')

            cell_label.grid(row=4, column=1, columnspan=2, sticky='w')
            cell_path_entry.grid(row=5, column=1, columnspan=2, sticky='w')
            cell_browse_button.grid(row=6, column=1, columnspan=2, sticky='w')

            gene_label.grid(row=7, column=1, columnspan=2, sticky='w')
            gene_path_entry.grid(row=8, column=1, columnspan=2, sticky='w')
            gene_browse_button.grid(row=9, column=1, columnspan=2, sticky='w')



            exclude_gene_label.grid(row=10, column=1, columnspan=2, sticky='w')
            exclude_gene_path_entry.grid(row=11, column=1, columnspan=2, sticky='w')
            exclude_gene_browse_button.grid(row=12, column=1, columnspan=2, sticky='w')

            annotate_gene_subset_label.grid(row=9, column=4, columnspan=2, sticky='w')
            annotate_gene_subset_path_entry.grid(row=10, column=4, columnspan=2, sticky='w')
            annotate_gene_subset_browse_button.grid(row=11, column=4, columnspan=2, sticky='w')

            gene_number_label.grid(row=13, column=1, columnspan=2, sticky='w')
            gene_number_entry.grid(row=14, column=1, columnspan=2, sticky='w')

            depth_number_label.grid(row=15, column=1, columnspan=2, sticky='w')
            depth_number_entry.grid(row=16, column=1, columnspan=2, sticky='w')

            for i, flag in enumerate(self.flags):
                var = tk.BooleanVar()
                tk.Checkbutton(self, text=flag, variable=var).grid(row=1+i, column=3, columnspan=1, sticky='w')
                self.variables.append(var)

            metric_menu_label.grid(row=2+len(self.flags), column=3, columnspan=1, sticky='w')
            metric_option_menu.grid(row=3+len(self.flags), column=3, columnspan=1, sticky='w')

            method_menu_label.grid(row=4+len(self.flags), column=3, columnspan=1, sticky='w')
            method_option_menu.grid(row=5+len(self.flags), column=3, columnspan=1, sticky='w')

            qgraph_menu_label.grid(row=15, column=4, columnspan=1, sticky='w')
            qgraph_option_menu.grid(row=16, column=4, columnspan=1, sticky='w')

            zdir_menu_label.grid(row=17, column=4, columnspan=1, sticky='w')
            zdir_option_menu.grid(row=18, column=4, columnspan=1, sticky='w')

            kmeans_range_label.grid(row=12, column=4, columnspan=1, sticky='w')
            kmeans_range_entry.grid(row=13, column=4, columnspan=1, sticky='w')

            color_cells_label.grid(row=1, column=4, columnspan=1, sticky='w')
            color_cells_entry.grid(row=2, column=4, columnspan=1, sticky='w')

            color_genes_label.grid(row=3, column=4, columnspan=1, sticky='w')
            color_genes_entry.grid(row=4, column=4, columnspan=1, sticky='w')

            test_clust_stability_label.grid(row=5, column=4, columnspan=1, sticky='w')
            test_clust_stability_entry.grid(row=6, column=4, columnspan=1, sticky='w')

            genes_corr_label.grid(row=7, column=4, columnspan=1, sticky='w')
            genes_corr_entry.grid(row=8, column=4, columnspan=1, sticky='w')

            asset_label.grid(row=17, column=1, columnspan=1, sticky='w')
            asset_entry.grid(row=18, column=1, columnspan=1, sticky='w')

            create_button.grid(row=24, column=2, columnspan=2)


        def browse(self):
            file_path= askopenfilename(**self.file_opt)
            if file_path:
                self.path.set(file_path)
        def browse_cellp(self):
            file_path= askopenfilename(**self.file_opt)
            if file_path:
                self.cell_path.set(file_path)
        def browse_genep(self):
            file_path= askopenfilename(**self.file_opt)
            if file_path:
                self.gene_path.set(file_path)
        def browse_excludeg(self):
            file_path= askopenfilename(**self.file_opt)
            if file_path:
                self.exclude_gene_path.set(file_path)
        def browse_annotateg(self):
            file_path= askopenfilename(**self.file_opt)
            if file_path:
                self.annotate_gene_subset.set(file_path)

        def genAsset(self):
            all_options_dict = {}
            asset_path = self.path.get()
            asset_name = self.asset.get()
            asset_metric_menu_option = self.metric_menu_var.get()
            asset_method_menu_option = self.method_menu_var.get()
            asset_gene_number = self.gene_number.get()
            asset_depth = self.depth_number.get()
            asset_cell_path = self.cell_path.get()
            asset_gene_path = self.gene_path.get()
            asset_zdir = self.zdir_menu_var.get()
            asset_qgraph = self.qgraph_menu_var.get()
            asset_kmeans_cluster_range = self.kmeans_cluster_range.get()
            asset_exclude_gene_path = self.exclude_gene_path.get()
            asset_color_cells = self.color_cells.get()
            asset_color_genes = self.color_genes.get()
            asset_test_clust_stability = self.test_clust_stability.get()
            asset_genes_corr = self.genes_corr.get()
            asset_annotate_gene_subset = self.annotate_gene_subset.get()
            for var, flag in zip(self.variables, self.flags):
                all_options_dict[flag] = var.get()
            all_options_dict['filepath'] = asset_path
            all_options_dict['output_name'] = asset_name
            all_options_dict['method'] = asset_method_menu_option
            all_options_dict['metric'] =asset_metric_menu_option
            all_options_dict['gene_number'] =asset_gene_number
            all_options_dict['depth'] = asset_depth
            all_options_dict['cell_file'] = asset_cell_path
            all_options_dict['gene_file'] = asset_gene_path
            all_options_dict['zdir'] = asset_zdir
            all_options_dict['qgraph'] = asset_qgraph
            all_options_dict['kmeans_cluster_range'] = asset_kmeans_cluster_range
            all_options_dict['exclude_genes'] = asset_exclude_gene_path
            all_options_dict['color_cells'] = asset_color_cells
            all_options_dict['color_genes'] = asset_color_genes
            all_options_dict['test_clust_stability'] = asset_test_clust_stability
            all_options_dict['genes_corr'] = asset_genes_corr
            all_options_dict['annotate_gene_subset'] = asset_annotate_gene_subset
            self.all_dict = all_options_dict
            self.destroy()
except ImportError:
    import Tkinter as tk
    import tkFileDialog.askopenfilename as askopenfilename

    class Window(tk.Frame):
        def __init__(self):
            tk.Frame.__init__(self)
            self.title("scicast")
            self.path = tk.StringVar()
            self.cell_path = tk.StringVar()
            self.gene_path = tk.StringVar()
            self.gene_label_path = tk.StringVar()
            self.exclude_gene_path = tk.StringVar()
            self.asset = tk.StringVar()
            self.gene_number = tk.IntVar(value=200)
            self.depth_number = tk.IntVar(value=20)
            self.kmeans_cluster_range = tk.StringVar(value='2,4')
            self.color_cells = tk.StringVar()
            self.color_genes = tk.StringVar()
            self.test_clust_stability = tk.IntVar(value=0)
            self.genes_corr = tk.StringVar()
            self.annotate_gene_subset = tk.StringVar()

            #type or choose gene matrix file
            dir_label = tk.Label(self, text="Browse or type path to gene cell matrix file:")
            path_entry = tk.Entry(self, textvariable=self.path, width=40)
            browse_button = tk.Button(self, text="Browse for gene cell matrix file", command=self.browse)

            #type or choose cell group file
            cell_label = tk.Label(self, text="Browse or type path to cell group file:")
            cell_path_entry = tk.Entry(self, textvariable=self.cell_path, width=40)
            cell_browse_button = tk.Button(self, text="Browse for cell group file", command=self.browse_cellp)

            #type or choose gene group file
            gene_label = tk.Label(self, text="Browse or type path to gene group file:")
            gene_path_entry = tk.Entry(self, textvariable=self.gene_path, width=40)
            gene_browse_button = tk.Button(self, text="Browse for gene group file", command=self.browse_genep)

            #type or choose file of genes to exclude from all analysis
            exclude_gene_label = tk.Label(self, text="Browse or type path to file with genes to exclude from analysis (i.e cell cycle):")
            exclude_gene_path_entry = tk.Entry(self, textvariable=self.exclude_gene_path, width=40)
            exclude_gene_browse_button = tk.Button(self, text="Browse for exclude genes file", command=self.browse_excludeg)

            #type or choose file of genes which will be labeled in gene PCA plot
            annotate_gene_subset_label = tk.Label(self, text="Annotate only these genes in gene PCA:")
            annotate_gene_subset_path_entry = tk.Entry(self, textvariable=self.annotate_gene_subset, width=40)
            annotate_gene_subset_browse_button = tk.Button(self, text="Browse gene annotation file", command=self.browse_annotateg)

            #define file extensions
            self.file_opt = options = {}
            options['defaultextension'] = '.txt'
            options['filetypes'] = [('all files', '.*'), ('text files', '.txt'),('csv files', '.csv')]

            #setup metric menu options
            self.metric_menu_var = tk.StringVar()
            self.metric_menu_var.set("seuclidean")
            metric_menu_label = tk.Label(self, text="Choose Metric:")
            metric_option_menu = tk.OptionMenu(self, self.metric_menu_var, 'braycurtis', 'canberra', 'chebyshev', 'cityblock', 'correlation', 'cosine', 'dice', 'euclidean', 'hamming', 'jaccard', 'kulsinski', 'mahalanobis', 'matching', 'minkowski', 'rogerstanimoto', 'russellrao', 'sokalmichener', 'sokalsneath', 'sqeuclidean', 'yule')

            #setup method option menu
            self.method_menu_var = tk.StringVar()
            self.method_menu_var.set("ward")
            method_menu_label = tk.Label(self, text="Choose Method:")
            method_option_menu = tk.OptionMenu(self, self.method_menu_var, 'single', 'complete', 'average', 'weighted', 'centroid', 'median')

            #setup qgraph option menu
            self.qgraph_menu_var = tk.StringVar()
            self.qgraph_menu_var.set("none")
            qgraph_menu_label = tk.Label(self, text="Choose which qgraph networks to generate:")
            qgraph_option_menu = tk.OptionMenu(self, self.qgraph_menu_var, 'gene','cell','both')

            #setup z-direction option menu
            self.zdir_menu_var = tk.IntVar()
            self.zdir_menu_var.set(0)
            zdir_menu_label = tk.Label(self, text="Choose z:")
            zdir_option_menu = tk.OptionMenu(self, self.zdir_menu_var, 1,'None')

            self.flags = ["Don't Run Heatmaps","Don't Run Correlation", "Verbose", "Test Significance by Groups (User Defined)", "Test Significance by Unbiased Clusters", "Exclude Cells Not in User Cell Groups", "Add Ellipse", "Add Cell Names to PCA", "Display Only Unique Signifcant Genes", "Run Significance Test for kmeans clusters", "Input Matrix is already log2"]
            self.variables = []



            asset_label = tk.Label(self, text="Output File Name:")
            asset_entry = tk.Entry(self, textvariable=self.asset, width=40)

            gene_number_label = tk.Label(self, text="Number of genes to include")
            gene_number_entry = tk.Entry(self, textvariable=self.gene_number, width=10)

            kmeans_range_label = tk.Label(self, text="Range of cluster for kmeans (inclusive):")
            kmeans_range_entry = tk.Entry(self, textvariable=self.kmeans_cluster_range, width=10)

            depth_number_label = tk.Label(self, text="Depth at which subclustering will stop")
            depth_number_entry = tk.Entry(self, textvariable=self.depth_number, width=10)

            color_cells_label = tk.Label(self, text="Provide specific colors and markers for each cell group.")
            color_cells_entry = tk.Entry(self, textvariable=self.color_cells, width=20)

            color_genes_label = tk.Label(self, text="Provide specific colors and markers for each gene group.")
            color_genes_entry = tk.Entry(self, textvariable=self.color_cells, width=20)

            test_clust_stability_label = tk.Label(self, text="Number of iterations to test cluster stability with varying gene numbers.")
            test_clust_stability_entry = tk.Entry(self, textvariable=self.test_clust_stability, width=10)

            genes_corr_label = tk.Label(self, text="Comma seperated list of genes to add to correlation search.")
            genes_corr_entry = tk.Entry(self, textvariable=self.genes_corr, width=20)


            create_button = tk.Button(self, text="Run scicast", command=self.genAsset)

            dir_label.grid(row=1, column=1, columnspan=2, sticky='w')
            path_entry.grid(row=2, column=1, columnspan=2, sticky='w')
            browse_button.grid(row=3, column=1, columnspan=2, sticky='w')

            cell_label.grid(row=4, column=1, columnspan=2, sticky='w')
            cell_path_entry.grid(row=5, column=1, columnspan=2, sticky='w')
            cell_browse_button.grid(row=6, column=1, columnspan=2, sticky='w')

            gene_label.grid(row=7, column=1, columnspan=2, sticky='w')
            gene_path_entry.grid(row=8, column=1, columnspan=2, sticky='w')
            gene_browse_button.grid(row=9, column=1, columnspan=2, sticky='w')



            exclude_gene_label.grid(row=10, column=1, columnspan=2, sticky='w')
            exclude_gene_path_entry.grid(row=11, column=1, columnspan=2, sticky='w')
            exclude_gene_browse_button.grid(row=12, column=1, columnspan=2, sticky='w')

            annotate_gene_subset_label.grid(row=9, column=4, columnspan=2, sticky='w')
            annotate_gene_subset_path_entry.grid(row=10, column=4, columnspan=2, sticky='w')
            annotate_gene_subset_browse_button.grid(row=11, column=4, columnspan=2, sticky='w')

            gene_number_label.grid(row=13, column=1, columnspan=2, sticky='w')
            gene_number_entry.grid(row=14, column=1, columnspan=2, sticky='w')

            depth_number_label.grid(row=15, column=1, columnspan=2, sticky='w')
            depth_number_entry.grid(row=16, column=1, columnspan=2, sticky='w')

            for i, flag in enumerate(self.flags):
                var = tk.BooleanVar()
                tk.Checkbutton(self, text=flag, variable=var).grid(row=1+i, column=3, columnspan=1, sticky='w')
                self.variables.append(var)

            metric_menu_label.grid(row=2+len(self.flags), column=3, columnspan=1, sticky='w')
            metric_option_menu.grid(row=3+len(self.flags), column=3, columnspan=1, sticky='w')

            method_menu_label.grid(row=4+len(self.flags), column=3, columnspan=1, sticky='w')
            method_option_menu.grid(row=5+len(self.flags), column=3, columnspan=1, sticky='w')

            qgraph_menu_label.grid(row=15, column=4, columnspan=1, sticky='w')
            qgraph_option_menu.grid(row=16, column=4, columnspan=1, sticky='w')

            zdir_menu_label.grid(row=17, column=4, columnspan=1, sticky='w')
            zdir_option_menu.grid(row=18, column=4, columnspan=1, sticky='w')

            kmeans_range_label.grid(row=12, column=4, columnspan=1, sticky='w')
            kmeans_range_entry.grid(row=13, column=4, columnspan=1, sticky='w')

            color_cells_label.grid(row=1, column=4, columnspan=1, sticky='w')
            color_cells_entry.grid(row=2, column=4, columnspan=1, sticky='w')

            color_genes_label.grid(row=3, column=4, columnspan=1, sticky='w')
            color_genes_entry.grid(row=4, column=4, columnspan=1, sticky='w')

            test_clust_stability_label.grid(row=5, column=4, columnspan=1, sticky='w')
            test_clust_stability_entry.grid(row=6, column=4, columnspan=1, sticky='w')

            genes_corr_label.grid(row=7, column=4, columnspan=1, sticky='w')
            genes_corr_entry.grid(row=8, column=4, columnspan=1, sticky='w')

            asset_label.grid(row=17, column=1, columnspan=1, sticky='w')
            asset_entry.grid(row=18, column=1, columnspan=1, sticky='w')

            create_button.grid(row=24, column=2, columnspan=2)


        def browse(self):
            file_path= askopenfilename(**self.file_opt)
            if file_path:
                self.path.set(file_path)
        def browse_cellp(self):
            file_path= askopenfilename(**self.file_opt)
            if file_path:
                self.cell_path.set(file_path)
        def browse_genep(self):
            file_path= askopenfilename(**self.file_opt)
            if file_path:
                self.gene_path.set(file_path)
        def browse_excludeg(self):
            file_path= askopenfilename(**self.file_opt)
            if file_path:
                self.exclude_gene_path.set(file_path)
        def browse_annotateg(self):
            file_path= askopenfilename(**self.file_opt)
            if file_path:
                self.annotate_gene_subset.set(file_path)

        def genAsset(self):
            all_options_dict = {}
            asset_path = self.path.get()
            asset_name = self.asset.get()
            asset_metric_menu_option = self.metric_menu_var.get()
            asset_method_menu_option = self.method_menu_var.get()
            asset_gene_number = self.gene_number.get()
            asset_depth = self.depth_number.get()
            asset_cell_path = self.cell_path.get()
            asset_gene_path = self.gene_path.get()
            asset_zdir = self.zdir_menu_var.get()
            asset_qgraph = self.qgraph_menu_var.get()
            asset_kmeans_cluster_range = self.kmeans_cluster_range.get()
            asset_exclude_gene_path = self.exclude_gene_path.get()
            asset_color_cells = self.color_cells.get()
            asset_color_genes = self.color_genes.get()
            asset_test_clust_stability = self.test_clust_stability.get()
            asset_genes_corr = self.genes_corr.get()
            asset_annotate_gene_subset = self.annotate_gene_subset.get()
            for var, flag in zip(self.variables, self.flags):
                all_options_dict[flag] = var.get()
            all_options_dict['filepath'] = asset_path
            all_options_dict['output_name'] = asset_name
            all_options_dict['method'] = asset_method_menu_option
            all_options_dict['metric'] =asset_metric_menu_option
            all_options_dict['gene_number'] =asset_gene_number
            all_options_dict['depth'] = asset_depth
            all_options_dict['cell_file'] = asset_cell_path
            all_options_dict['gene_file'] = asset_gene_path
            all_options_dict['zdir'] = asset_zdir
            all_options_dict['qgraph'] = asset_qgraph
            all_options_dict['kmeans_cluster_range'] = asset_kmeans_cluster_range
            all_options_dict['exclude_genes'] = asset_exclude_gene_path
            all_options_dict['color_cells'] = asset_color_cells
            all_options_dict['color_genes'] = asset_color_genes
            all_options_dict['test_clust_stability'] = asset_test_clust_stability
            all_options_dict['genes_corr'] = asset_genes_corr
            all_options_dict['annotate_gene_subset'] = asset_annotate_gene_subset
            self.all_dict = all_options_dict
            self.destroy()



import numpy as np
import pandas as pd
import os
import matplotlib
matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
from matplotlib.ticker import LinearLocator
import seaborn as sns
from operator import itemgetter
import matplotlib.ticker as ticker
import math
import matplotlib.patches as patches



#run correlation matrix and save only those above threshold
def run_corr(args, df_by_gene, title, path_filename, method_name='pearson', sig_threshold= 0.5, min_period=3, save_corrs=False):
    try:
        from .dim_reduction import return_top_pca_gene
    except SystemError:
        from dim_reduction import return_top_pca_gene

    if len(df_by_gene.columns.tolist())>5000:
        df_by_gene, top_pca_list = return_top_pca_gene(args, df_by_gene.transpose(), user_num_genes=5000)
    if method_name != 'kendall':
        corr_by_gene = df_by_gene.corr(method=method_name, min_periods=min_period)
    else:
        corr_by_gene = df_by_gene.corr(method=method_name)


    cor = corr_by_gene
    cor.loc[:,:] =  np.tril(cor.values, k=-1)
    cor = cor.stack()
    corr_by_gene_pos = cor[cor >=sig_threshold]
    corr_by_gene_neg = cor[cor <=(sig_threshold*-1)]

    cor_pos_df = pd.DataFrame(corr_by_gene_pos)
    cor_neg_df = pd.DataFrame(corr_by_gene_neg)
    sig_corr = cor_pos_df.append(cor_neg_df)
    sig_corrs = pd.DataFrame(sig_corr[0], columns=["corr"])


    sig_corrs.to_csv(os.path.join(path_filename, title+'_counts_corr_sig_'+method_name+'.txt'), sep = '\t')
    return sig_corrs

#finds most correlated gene groups that are not overlapping
def find_top_corrs(terms_to_search, sig_corrs, num_to_return, gene_corr_list = []):
    all_corrs_list = []
    best_corrs_list = []
    for term_to_search in terms_to_search:
        corr_tup = [(term_to_search, 1)]
        for index, row in sig_corrs.iterrows():
            if term_to_search in index:
                if index[0]==term_to_search:
                    corr_tup.append((index[1],row['corr']))
                else:
                    corr_tup.append((index[0],row['corr']))
        all_corrs_list.append(corr_tup)
    all_corrs_list.sort(key=len, reverse=True)
    good_count = 0
    len_count = 0
    corr_genes_seen = []
    while good_count <= num_to_return and len_count <= len(all_corrs_list):
        for i, corrs in enumerate(all_corrs_list):
            len_count+=1
            if corrs[0][0] not in corr_genes_seen:
                best_corrs_list.append(corrs)
                good_count+=1
            for g, c in corrs:
                if g not in corr_genes_seen and '-' not in str(c):
                    corr_genes_seen.append(g)
    if gene_corr_list != []:
        search_corrs = []
        for term in gene_corr_list:
            corr_tup = [(term, 1)]
            for index, row in sig_corrs.iterrows():
                if term in index:
                    if index[0]==term:
                        corr_tup.append((index[1],row['corr']))
                    else:
                        corr_tup.append((index[0],row['corr']))
            search_corrs.append(corr_tup)
        best_corrs_list = search_corrs+best_corrs_list
        return best_corrs_list[0:num_to_return+len(gene_corr_list)+1]
    else:
        return best_corrs_list[0:num_to_return]


#corr_plot finds and plots all correlated genes, log turns on log scale, sort plots the genes in the rank order of the gene searched
def corr_plot(terms_to_search, df_by_gene_corr, args, matrix_data, title ='', sort=True, sig_threshold=0.5):
    path_filename = matrix_data.new_filepath

    #if there are genes supplied with genes_corr flag process them to a list for correlation search
    if args.genes_corr != '':
        gene_corr_list = args.genes_corr.split(',')
    #otherwise pass an empty list
    else:
        gene_corr_list = []
    size_cells = len(df_by_gene_corr.index.tolist())
    figlen=int(size_cells/11)
    if figlen < 15:
        figlen = 15
    ncol = int(figlen/3.2)

    if size_cells <100:
        sig_threshold = -0.137*math.log(size_cells)+1.1322
    sig_corrs = run_corr(args, df_by_gene_corr, title, path_filename, sig_threshold=sig_threshold)

    corr_list = find_top_corrs(terms_to_search, sig_corrs, num_to_return=3, gene_corr_list=gene_corr_list)

    for corr_tup in corr_list:
        term_to_search = corr_tup[0][0]
        corr_tup.sort(key=itemgetter(1), reverse=True)
        corr_df = pd.DataFrame(corr_tup, columns=['GeneID', 'Correlation'])
        corr_df.to_csv(os.path.join(matrix_data.new_filepath, title+'_Corr_w_'+term_to_search+'_list.txt'), sep = '\t', index=False)

        to_plot = [x[0] for x in corr_tup]
        sns.set_palette(sns.cubehelix_palette(len(to_plot), start=1, rot=-.9, reverse=True))
        sns.set_context("notebook", font_scale=.9, rc={"lines.linewidth": 1})
        try:
            sorted_df = df_by_gene_corr.sort_values(by=[term_to_search])
            ylabel='Counts (log2)'
            if sort:
                ax = sorted_df[to_plot].plot(figsize = (figlen,10))
                xlabels = sorted_df[to_plot].index.values
            else:
                ax = df_by_gene_corr[to_plot].plot(figsize = (figlen,10))
                xlabels = df_by_gene_corr[to_plot].index.values
            ax.set_xlabel('Cell Label')
            ax.set_ylabel(ylabel)
            ax.set_title('Correlates with '+term_to_search, loc='right')
            ax.xaxis.set_minor_locator(LinearLocator(numticks=len(xlabels)))
            if matrix_data.cell_label_map:
                ax.set_xticklabels(xlabels, minor=True, rotation='vertical', fontsize=3)
                Xcolors = [matrix_data.cell_label_map[cell][0][0] for cell in xlabels]
                group_labels = [matrix_data.cell_label_map[cell][0][2] for cell in xlabels]
                group_seen = []
                leg_handles = []
                for xtick, xcolor, group_name in zip(ax.get_xticklabels(which='minor'), Xcolors, group_labels):
                    xtick.set_color(xcolor)
                    xtick.set_rotation(90)
                    if group_name not in group_seen:
                        leg_handles.append(patches.Patch(color=xcolor, label=group_name))
                        group_seen.append(group_name)

            else:
                ax.set_xticklabels(xlabels, minor=True, rotation='vertical', fontsize=3)
            ax.set_ylim([0, df_by_gene_corr[to_plot].values.max()])
            ax.xaxis.set_major_formatter(ticker.NullFormatter())
            ax.tick_params(axis='x', which ='minor', labelsize=9)
            #scale bbox anchoring to account for number of correlated genes and plot size
            if len(corr_tup)>1:
                bbox_height = float(1E-13)*pow(len(corr_tup),6) - float(7E-11)*pow(len(corr_tup),5) + float(1E-8)*pow(len(corr_tup),4) - float(8E-7)*pow(len(corr_tup),3) - float(3E-5)*pow(len(corr_tup),2) + 0.0086*len(corr_tup) + 1.0042
            else:
                bbox_height = 1.05
            l_labels = [str(x[0])+' '+"%.2f" % x[1] for x in corr_tup]
            if matrix_data.cell_label_map:
                first_legend = ax.legend(l_labels, loc='upper left', bbox_to_anchor=(0.01, bbox_height+.1), ncol=ncol, prop={'size':10})
                ax = plt.gca().add_artist(first_legend)
                plt.legend(handles=leg_handles, loc='upper right', bbox_to_anchor=(0.9, bbox_height+.1))
            else:
                ax.legend(l_labels, loc='upper left', bbox_to_anchor=(0.01, bbox_height), ncol=ncol, prop={'size':10})
            fig = plt.gcf()
            fig.subplots_adjust(bottom=0.08, top=0.95, right=0.98, left=0.03)
            plt.savefig(os.path.join(path_filename, title+'_corr_with_'+term_to_search+'.pdf'), bbox_inches='tight')
            plt.close('all')
        except KeyError:
            if args.verbose:
                print(term_to_search+' not in this matrix.')
            pass





def is_valid_file(parser, arg):
    import os
    """
    Check if arg is a valid file that already exists on the file system.

    Parameters
    ----------
    parser : argparse object
    arg : str

    Returns
    -------
    arg
    """
    arg = os.path.abspath(arg)
    if not os.path.exists(arg):
        parser.error("The file %s does not exist!" % arg)
    else:
        return arg

from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter,SUPPRESS
import sys as _sys
from gettext import gettext as _
class MyParser(ArgumentParser):
    def error(self, message):
        usage = self.usage
        self.usage = None
        self.print_usage(_sys.stderr)
        self.exit(2, _('%s: error: %s\n') % (self.prog, message))
        self.usage = usage
def check_gui_parser():
    p = MyParser(description=__doc__,formatter_class=ArgumentDefaultsHelpFormatter, usage=SUPPRESS)
    p.add_argument("-gui",
                        dest="gui_true",
                        default = False,
                        action = "store_true",
                        help="To run gui instead of command line.")
    gui_args = p.parse_args()
    return(gui_args)

def get_parser():
    """Get parser object for script cluster.py."""
    from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
    parser = ArgumentParser(description=__doc__,
                            formatter_class=ArgumentDefaultsHelpFormatter)
    parser.add_argument("-f", "-filepath",
                        dest="filepath",
                        type=lambda x: is_valid_file(parser, x),
                        required =True,
                        help="Filepath to cell-gene matrix file")
    parser.add_argument("-n","-name",
                        dest="base_name",
                        default='scicast_analysis',
                        type=str,
                        help="The base name for files that will be generated.")
    parser.add_argument("-g","-gene_number",
                        dest="gene_number",
                        default=200,
                        type=int,
                        help="The number of genes that will be selected from top pca genes on each iteration.")
    parser.add_argument("-v", "-verbose",
                        action="store_true",
                        dest="verbose",
                        default=False,
                        help="Print more")
    parser.add_argument("-cell_group",
                        dest="cell_list_filename",
                        default=False,
                        help="Provide a filename with two columns: 'SampleID' and 'GroupID'. If GroupID is empty the SampleID list will be used to restrict cells prior to analysis.")
    parser.add_argument("-gene_list",
                        dest="gene_list_filename",
                        default=False,
                        type=str,
                        help="Path to a file with a two columns with headers 'GeneID' and 'GroupID' (Singular format). GeneID list will be used to create a new matrix file with only those genes included.")
    parser.add_argument("-group_sig_test",
                        action="store_true",
                        default=False,
                        dest="group_sig_test",
                        help="If cell groups are provided, perform significance testing between all groups (independent of any cluster groups).")
    parser.add_argument("-stability",
                        dest="test_clust_stability",
                        type=int,
                        default=0,
                        help="Provide a number of iterations to test how stable clustering is as the number of top PCA genes changes from 100-1000. Output will be clustering heatmaps for each iteration a summary of changes as gene number varies.")
    parser.add_argument("-metric",
                        dest="metric",
                        default='euclidean',
                        type=str,
                        help="The distance metric to use. The distance function can be: braycurtis, canberra, chebyshev, cityblock, correlation, cosine, dice, euclidean, hamming, jaccard, kulsinski, mahalanobis, matching, minkowski, rogerstanimoto, russellrao, seuclidean, sokalmichener, sokalsneath, sqeuclidean, yule.")
    parser.add_argument("-method",
                        dest="method",
                        default='weighted',
                        type=str,
                        help="The linkage method to use. The linkage algorithm can be: single, complete, average, weighted, centroid, median or ward.")
    parser.add_argument("-depth",
                        dest="depth_of_clustering",
                        type=int,
                        default=20,
                        help="The size in cell number at which sub-clustering analysis will stop clustering, pca and correlation analysis.")
    parser.add_argument("-genes_corr",
                        dest="genes_corr",
                        default='',
                        type=str,
                        help="If you want to look for correlation on a specific gene or set of genes enter them as a comma seperated list i.e. 'Gapdh,Actb'.")
    parser.add_argument("-all_sig",
                        dest="all_sig",
                        action="store_true",
                        help="Do significance testing on all hierarchical clustering groups. The minimum number of cells in a group is set by --depth.")
    parser.add_argument("-already_log2",
                        dest="already_log2",
                        default=False,
                        action="store_true",
                        help="If the gene matrix is already log2 transformed i.e. rld or vsd DESeq2, this will disable log2 transform.")
    parser.add_argument("-z",
                        dest="z_direction",
                        default=0,
                        help="Either 0 (rows) or 1 (columns) or None. Whether or not to calculate z-scores for the rows or the columns. Z scores are: z = (x - mean)/std, so values in each row (column) will get the mean of the row (column) subtracted, then divided by the standard deviation of the row (column). This ensures that each row (column) has mean of 0 and variance of 1.")
    parser.add_argument("-no_corr",
                        dest="no_corr",
                        action="store_true",
                        default=False,
                        help="Don't run correlation search. Default is on.")
    parser.add_argument("-no_heatmaps",
                        dest="no_heatmaps",
                        action="store_true",
                        default=False,
                        help="Don't run heatmaps and pca. Default is will generate heatmaps and pca.")
    parser.add_argument("-exclude_genes",
                        dest="exclude_genes",
                        default=False,
                        type=str,
                        help="Filepath to list of genes to be excluded from analysis (at all levels of analysis). Header must be 'GeneID'.")
    parser.add_argument("-limit_cells",
                        dest="limit_cells",
                        action="store_true",
                        default=False,
                        help="With cell group file, will exclude all other cells from analysis (at all levels of analysis). Header must be 'SampleID'.")
    parser.add_argument("-add_ellipse",
                        dest="add_ellipse",
                        action="store_true",
                        default=False,
                        help="When present colored ellipses will be added to cell and gene PCA plots. Must provide gene and/or cell groups.")
    parser.add_argument("-annotate_gene_subset",
                        dest="annotate_gene_subset",
                        default=False,
                        help="Provide path or filename (if in same file) to file with genes to be annotated on gene PCA. Must have 'GeneID' header.")
    parser.add_argument("-annotate_cell_pca",
                        dest="annotate_cell_pca",
                        action="store_true",
                        default=False,
                        help="Option will annotate cell PCA with cell names. Default is off (False).")
    parser.add_argument("-color_cells",
                        dest="color_cells",
                        default=False,
                        type=str,
                        help="Follow with cell group names and pairs (matching cell group names). Forces color and marker style on group. Just color or color and marker can be provided.  Example: Group1,color,marker Group2,red,v")
    parser.add_argument("-color_genes",
                        dest="color_genes",
                        default=False,
                        type=str,
                        help="Can be 'same' if names are the same as cell groups. Follow with gene group names and pairs (matching gene group names). Forces color and marker style on group. Just color or color and marker can be provided.  Example: Group1,color,marker Group2,red,v")
    parser.add_argument("-qgraph_plot",
                        dest="qgraph_plot",
                        default=False,
                        type=str,
                        help="Can be 'cell' 'gene' or 'both' will plot qgraph gene and/or cell network and pca correlation network plot to pdf. Requires both gene and cell groups to be provided.")
    parser.add_argument("-sig_unique",
                        dest="sig_unique",
                        action="store_false",
                        default=True,
                        help="group_sig_test will by default find unique sets of significant genes, so that the whole list has no duplicates. This switches the top significant genes amoung groups to allow repeats.")
    parser.add_argument("-kmeans_cluster_range",
                        dest="kmeans_cluster_range",
                        type=str,
                        default='2,4',
                        help="range of cluster sizes to run kmeans clustering (inclusive)")
    parser.add_argument("-kmeans_sig_test",
                        dest="kmeans_sig_test",
                        default=False,
                        action='store_true',
                        help="Run significance test between groups defined by kmeans clusters.")
    args = parser.parse_args()
    yes_or_no_options = ["Don't Run Heatmaps", "Don't Run Correlation", "Verbose", "Test Significance by Groups (User Defined)", "Test Significance by Unbiased Clusters", "Exclude Cells Not in User Cell Groups", "Add Ellipse", "Add Cell Names to PCA", "Display Only Unique Signifcant Genes", "Run Significance Test for kmeans clusters", "Input Matrix is already log2"]
    yes_or_no_answers = [args.no_heatmaps,args.no_corr, args.verbose,args.group_sig_test, args.all_sig, args.limit_cells, args.add_ellipse, args.annotate_cell_pca, args.sig_unique, args.kmeans_sig_test, args.already_log2]

    all_options_dict = {}
    all_options_dict['filepath'] = args.filepath
    all_options_dict['output_name'] = args.base_name
    all_options_dict['method'] = args.method
    all_options_dict['metric'] =args.metric
    all_options_dict['gene_number'] =args.gene_number
    all_options_dict['depth'] = args.depth_of_clustering
    all_options_dict['cell_file'] = args.cell_list_filename
    all_options_dict['gene_file'] = args.gene_list_filename
    all_options_dict['zdir'] = args.z_direction
    all_options_dict['qgraph'] = args.qgraph_plot
    all_options_dict['kmeans_cluster_range'] = args.kmeans_cluster_range
    all_options_dict['exclude_genes'] = args.exclude_genes
    all_options_dict['already_log2'] = args.already_log2
    all_options_dict['color_cells'] = args.color_cells
    all_options_dict['color_genes'] = args.color_genes
    all_options_dict['test_clust_stability'] = args.test_clust_stability
    all_options_dict['genes_corr'] = args.genes_corr
    all_options_dict['annotate_gene_subset'] = args.annotate_gene_subset
    for var, flag in zip(yes_or_no_answers, yes_or_no_options):
        all_options_dict[flag] = var
    try:
        from .sci_load import Sci_load
    except SystemError:
        from sci_load import Sci_load
    scil = Sci_load()
    opts_all = scil.load_options(all_options_dict = all_options_dict)

    return opts_all

if __name__ == "__main__":
    pass



import numpy as np
import pandas as pd
import os
import matplotlib
matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
from sklearn.decomposition import PCA as skPCA
import seaborn as sns
import matplotlib.patches as patches
from sklearn.manifold import TSNE
from sklearn.decomposition import TruncatedSVD
from sklearn.cluster import KMeans



def ellip_enclose(points, color, inc=1, lw=2, nst=2):
    """
    Plot the minimum ellipse around a set of points.

    Based on:
    https://github.com/joferkington/oost_paper_code/blob/master/error_ellipse.py
    """

    def eigsorted(cov):
        vals, vecs = np.linalg.eigh(cov)
        order = vals.argsort()[::-1]
        return vals[order], vecs[:,order]

    x = points[:,0]
    y = points[:,1]
    cov = np.cov(x, y)
    vals, vecs = eigsorted(cov)
    theta = np.degrees(np.arctan2(*vecs[:,0][::-1]))
    w, h = 2 * nst * np.sqrt(vals)
    center = np.mean(points, 0)
    ell = patches.Ellipse(center, width=inc*w, height=inc*h, angle=theta,
                          facecolor=color, alpha=0.2, lw=0)
    edge = patches.Ellipse(center, width=inc*w, height=inc*h, angle=theta,
                          facecolor='none', edgecolor=color, lw=lw)
    return ell, edge

def return_top_pca_gene(args, by_cell_matrix, user_num_genes = None):
    gene_pca = skPCA(n_components=3)
    np_by_gene = np.asarray(by_cell_matrix.transpose())
    gene_index = by_cell_matrix.index.tolist()

    if user_num_genes is not None:
        num_genes = user_num_genes
    else:
        num_genes = min(args.gene_number, len(gene_index))
    by_gene_trans = gene_pca.fit_transform(np_by_gene)
    Pc_df = pd.DataFrame(gene_pca.components_.T, columns=['PC-1', 'PC-2', 'PC-3'], index=gene_index)
    pca_rank_df = Pc_df.abs().sum(axis=1)
    Pc_sort_df = pca_rank_df.nlargest(len(gene_index))
    top_pca_list = Pc_sort_df.index.tolist()
    new_cell_matrix = by_cell_matrix.ix[top_pca_list[0:num_genes],:]
    return new_cell_matrix.transpose(), top_pca_list[0:num_genes]

def plot_PCA(args, matrix_data, title = '', gene_subcluster_matrix=False):
    sns.set_palette("RdBu_r", 10, 1)
    if gene_subcluster_matrix:
        gene_list = gene_subcluster_matrix.columns.tolist()
        len_gene_list = len(gene_list)
        num_genes = min(args.gene_number, len_gene_list)

        df_by_gene = gene_subcluster_matrix.transpose()

    elif isinstance(matrix_data.log2_df_cell_gene_restricted,pd.DataFrame):
        gene_list = matrix_data.short_gene_list
        len_gene_list = len(gene_list)
        num_genes = min(args.gene_number, len_gene_list)

        df_by_gene = matrix_data.log2_df_cell_gene_restricted.transpose()

    else:
        gene_list = matrix_data.gene_list
        len_gene_list = len(gene_list)
        num_genes = min(args.gene_number, len_gene_list)

        df_by_gene = matrix_data.log2_df_cell.transpose()

    gene_pca = skPCA(n_components=3)
    np_by_gene = np.asarray(df_by_gene)

    by_gene_trans = gene_pca.fit_transform(np_by_gene)
    Pc_df = pd.DataFrame(gene_pca.components_.T, columns=['PC-1', 'PC-2', 'PC-3'], index=gene_list)
    pca_rank_df = Pc_df.abs().sum(axis=1)
    Pc_sort_df = pca_rank_df.nlargest(len_gene_list)
    top_pca_list = Pc_sort_df.index.tolist()

    top_by_gene = df_by_gene[top_pca_list[0:num_genes]]
    gene_top = skPCA(n_components=2)
    cell_pca = skPCA(n_components=2)
    top_by_cell = top_by_gene.transpose()
    np_top_gene = np.asarray(top_by_cell)
    np_top_cell = np.asarray(top_by_gene)
    top_cell_trans = cell_pca.fit_transform(np_top_cell)
    top_gene_trans = gene_top.fit_transform(np_top_gene)
    if not np.isnan(top_cell_trans).any():
        fig, (ax_cell, ax_gene) = plt.subplots(2, 1, figsize=(15, 30), sharex=False)
        rect_cell = ax_cell.patch
        rect_gene = ax_gene.patch
        rect_cell.set_facecolor('white')
        rect_gene.set_facecolor('white')
        ax_cell.grid(b=True, which='major', color='grey', linestyle='--', linewidth=0.3)
        ax_gene.grid(b=True, which='major', color='grey', linestyle='--', linewidth=0.3)
        if matrix_data.cell_label_map:
            annotate = args.annotate_cell_pca
            X = [x for x in top_cell_trans[:, 0]]
            Y = [y for y in top_cell_trans[:, 1]]
            for group_index , group_name in enumerate(matrix_data.cell_group_names):
                if group_index == 0:
                    labels = [matrix_data.cell_label_map[cell][group_index][2] for cell in top_by_cell.columns.tolist()]
                    colors = [matrix_data.cell_label_map[cell][group_index][0] for cell in top_by_cell.columns.tolist()]
                elif group_index > 0:
                    labels = [a+'_'+b for a, b in zip(labels, [matrix_data.cell_label_map[cell][group_index][2] for cell in top_by_cell.columns.tolist()])]

            markers = [matrix_data.cell_label_map[cell][0][1] for cell in top_by_cell.columns.tolist()]

            label_done = []
            xy_by_color_dict = {}
            for c in set(colors):
                xy_by_color_dict[c] = []
            for X_pos, Y_pos, m, color, l in zip(X, Y, markers, colors, labels):
                if l in label_done:
                    lab = ''
                else:
                    lab= l
                    label_done.append(l)
                xy_by_color_dict[color].append([X_pos,Y_pos])
                ax_cell.scatter(X_pos, Y_pos, marker=m, c=color, label=lab, s=30)
            if args.add_ellipse:
                for c in set(colors):
                    ell, edge = ellip_enclose(np.asarray(xy_by_color_dict[c]), c)
                    ax_cell.add_artist(ell)
                    ax_cell.add_artist(edge)
        else:
            ax_cell.scatter(top_cell_trans[:, 0], top_cell_trans[:, 1], alpha=0.75)
            annotate = args.annotate_cell_pca
        ax_cell.set_xlim([min(top_cell_trans[:, 0])-1, max(top_cell_trans[:, 0]+1)])
        ax_cell.set_ylim([min(top_cell_trans[:, 1])-1, max(top_cell_trans[:, 1]+2)])
        ax_cell.set_title(title+'_cell')
        if matrix_data.cell_label_map:
            handles, labs = ax_cell.get_legend_handles_labels()
            # sort both labels and handles by labels
            labs, handles = zip(*sorted(zip(labs, handles), key=lambda t: t[0]))
            ax_cell.legend(handles, labs, loc='best', ncol=1, prop={'size':12}, markerscale=1.5, frameon=True)
        ax_cell.set_xlabel('PC1')
        ax_cell.set_ylabel('PC2')
        if annotate:
            for label, x, y in zip(top_by_cell.columns, top_cell_trans[:, 0], top_cell_trans[:, 1]):
                ax_cell.annotate(label, (x+0.1, y+0.1))

        if matrix_data.gene_label_map:
            X = [x for x in top_gene_trans[:, 0]]
            Y = [y for y in top_gene_trans[:, 1]]
            labels = top_by_gene.columns.tolist()
            markers = [matrix_data.gene_label_map[gene][1] for gene in top_by_gene.columns.tolist()]
            colors = [matrix_data.gene_label_map[gene][0] for gene in top_by_gene.columns.tolist()]
            xy_by_color_dict = {}
            for c in set(colors):
                xy_by_color_dict[c] = []
            for X_pos, Y_pos, m, color, l in zip(X, Y, markers, colors, labels):
                xy_by_color_dict[color].append([X_pos,Y_pos])
                ax_gene.scatter(X_pos, Y_pos, marker=m, c=color, label = l, s=30)
            if args.add_ellipse:
                for c in set(colors):
                    ell, edge = ellip_enclose(np.asarray(xy_by_color_dict[c]), c)
                    ax_gene.add_artist(ell)
                    ax_gene.add_artist(edge)

        else:
            ax_gene.scatter(top_gene_trans[:, 0], top_gene_trans[:, 1], alpha=0.75)
        ax_gene.set_xlim([min(top_gene_trans[:, 0])-1, max(top_gene_trans[:, 0])+1])
        ax_gene.set_ylim([min(top_gene_trans[:, 1])-1, max(top_gene_trans[:, 1])+2])
        ax_gene.set_title(title+'_gene')
        ax_gene.set_xlabel('PC1')
        ax_gene.set_ylabel('PC2')
        if args.annotate_gene_subset:
            plot_subset_path = os.path.join(os.path.dirname(args.filepath),args.annotate_gene_subset)
            genes_plot = pd.read_csv(plot_subset_path, sep='\t', index_col=False)
            for label, x, y in zip(top_by_gene.columns, top_gene_trans[:, 0], top_gene_trans[:, 1]):
                if label in genes_plot['GeneID'].tolist():
                    if '_' in label:
                        label = label.split('_')[0]
                    ax_gene.annotate(label, (x+0.1, y+0.1))
        else:
            for label, x, y in zip(top_by_gene.columns, top_gene_trans[:, 0], top_gene_trans[:, 1]):
                if '_' in label:
                    label = label.split('_')[0]
                ax_gene.annotate(label, (x+0.1, y+0.1))

        if title != '':
            save_name = '_'.join(title.split(' ')[0:2])
            plt.savefig(os.path.join(matrix_data.new_filepath,save_name+'_skpca.pdf'), bbox_inches='tight')
        else:
            plt.savefig(os.path.join(matrix_data.new_filepath,'Group0_skpca.pdf'), bbox_inches='tight')
        plt.close('all')
        return top_pca_list
    else:
        return []

def plot_SVD(args, matrix_data, gene_subcluster_matrix=False, title = ''):
    sns.set(context= 'poster', font_scale = 1.2)
    sns.set_palette("RdBu_r", 10, 1)
    if isinstance(gene_subcluster_matrix, pd.DataFrame):
        gene_list = gene_subcluster_matrix.columns.tolist()
        len_gene_list = len(gene_list)
        num_genes = min(args.gene_number, len_gene_list)

        df_by_gene = gene_subcluster_matrix

    elif isinstance(matrix_data.log2_df_cell_gene_restricted, pd.DataFrame):
        gene_list = matrix_data.short_gene_list
        len_gene_list = len(gene_list)
        num_genes = min(args.gene_number, len_gene_list)

        df_by_gene = matrix_data.log2_df_cell_gene_restricted.transpose()

    else:
        gene_list = [x for x in matrix_data.gene_list]
        len_gene_list = len(gene_list)
        num_genes = min(args.gene_number, len_gene_list)

        df_by_gene = matrix_data.log2_df_cell.transpose()
    gene_pca = TruncatedSVD(n_components=3)
    np_by_gene = np.asarray(df_by_gene)

    by_gene_trans = gene_pca.fit_transform(np_by_gene)

    Pc_df = pd.DataFrame(gene_pca.components_.T, columns=['PC-1', 'PC-2', 'PC-3'], index=gene_list)
    pca_rank_df = Pc_df.abs().sum(axis=1)
    Pc_sort_df = pca_rank_df.nlargest(len_gene_list)
    top_pca_list = Pc_sort_df.index.tolist()

    top_by_gene = df_by_gene[top_pca_list[0:num_genes]]
    gene_top = TruncatedSVD(n_components=2)
    cell_pca = TruncatedSVD(n_components=2)
    top_by_cell = top_by_gene.transpose()
    np_top_gene = np.asarray(top_by_cell)
    np_top_cell = np.asarray(top_by_gene)
    top_cell_trans = cell_pca.fit_transform(np_top_cell)
    top_gene_trans = gene_top.fit_transform(np_top_gene)
    if not np.isnan(top_cell_trans).any():
        fig, (ax_cell, ax_gene) = plt.subplots(2, 1, figsize=(15, 30), sharex=False)
        rect_cell = ax_cell.patch
        rect_gene = ax_gene.patch
        rect_cell.set_facecolor('white')
        rect_gene.set_facecolor('white')
        ax_cell.grid(b=True, which='major', color='grey', linestyle='--', linewidth=0.3)
        ax_gene.grid(b=True, which='major', color='grey', linestyle='--', linewidth=0.3)
        if matrix_data.cell_label_map:
            annotate = args.annotate_cell_pca
            X = [x for x in top_cell_trans[:, 0]]
            Y = [y for y in top_cell_trans[:, 1]]
            for group_index , group_name in enumerate(matrix_data.cell_group_names):
                if group_index == 0:
                    labels = [matrix_data.cell_label_map[cell][group_index][2] for cell in top_by_cell.columns.tolist()]
                    colors = [matrix_data.cell_label_map[cell][group_index][0] for cell in top_by_cell.columns.tolist()]
                elif group_index > 0:
                    labels = [a+'_'+b for a, b in zip(labels, [matrix_data.cell_label_map[cell][group_index][2] for cell in top_by_cell.columns.tolist()])]
            label_set = list(set(labels))
            markers_dict = {}
            for i, lab_set in enumerate(label_set):
                markers_dict[lab_set] = matrix_data.markers[i]
            markers = [markers_dict[x] for x in labels]

            label_done = []
            xy_by_color_dict = {}
            for c in set(colors):
                xy_by_color_dict[c] = []
            for X_pos, Y_pos, m, color, l in zip(X, Y, markers, colors, labels):
                if l in label_done:
                    lab = ''
                else:
                    lab= l
                    label_done.append(l)
                xy_by_color_dict[color].append([X_pos,Y_pos])
                ax_cell.scatter(X_pos, Y_pos, marker=m, c=color, label=lab, s=30)
            if args.add_ellipse:
                for c in set(colors):
                    ell, edge = ellip_enclose(np.asarray(xy_by_color_dict[c]), c)
                    ax_cell.add_artist(ell)
                    ax_cell.add_artist(edge)
        else:
            ax_cell.scatter(top_cell_trans[:, 0], top_cell_trans[:, 1], alpha=0.75)
            annotate = args.annotate_cell_pca
        ax_cell.set_xlim([min(top_cell_trans[:, 0])-1, max(top_cell_trans[:, 0]+1)])
        ax_cell.set_ylim([min(top_cell_trans[:, 1])-1, max(top_cell_trans[:, 1]+2)])
        ax_cell.set_title(title+'_cell')
        if matrix_data.cell_label_map:
            handles, labs = ax_cell.get_legend_handles_labels()
            # sort both labels and handles by labels
            labs, handles = zip(*sorted(zip(labs, handles), key=lambda t: t[0]))
            ax_cell.legend(handles, labs, loc='best', ncol=1, prop={'size':12}, markerscale=1.5, frameon=True)
        ax_cell.set_xlabel('PC1')
        ax_cell.set_ylabel('PC2')
        if annotate:
            for label, x, y in zip(top_by_cell.columns, top_cell_trans[:, 0], top_cell_trans[:, 1]):
                ax_cell.annotate(label, (x+0.1, y+0.1))

        if matrix_data.gene_label_map:
            X = [x for x in top_gene_trans[:, 0]]
            Y = [y for y in top_gene_trans[:, 1]]
            labels = top_by_gene.columns.tolist()
            markers = [matrix_data.gene_label_map[gene][1] for gene in top_by_gene.columns.tolist()]
            colors = [matrix_data.gene_label_map[gene][0] for gene in top_by_gene.columns.tolist()]
            xy_by_color_dict = {}
            for c in set(colors):
                xy_by_color_dict[c] = []
            for X_pos, Y_pos, m, color, l in zip(X, Y, markers, colors, labels):
                xy_by_color_dict[color].append([X_pos,Y_pos])
                ax_gene.scatter(X_pos, Y_pos, marker=m, c=color, label = l, s=30)
            if args.add_ellipse:
                for c in set(colors):
                    ell, edge = ellip_enclose(np.asarray(xy_by_color_dict[c]), c)
                    ax_gene.add_artist(ell)
                    ax_gene.add_artist(edge)

        else:
            ax_gene.scatter(top_gene_trans[:, 0], top_gene_trans[:, 1], alpha=0.75)
        ax_gene.set_xlim([min(top_gene_trans[:, 0])-1, max(top_gene_trans[:, 0])+1])
        ax_gene.set_ylim([min(top_gene_trans[:, 1])-1, max(top_gene_trans[:, 1])+2])
        ax_gene.set_title(title+'_gene')
        ax_gene.set_xlabel('PC1')
        ax_gene.set_ylabel('PC2')
        if args.annotate_gene_subset:
            plot_subset_path = os.path.join(os.path.dirname(args.filepath),args.annotate_gene_subset)
            genes_plot = pd.read_csv(plot_subset_path, sep='\t', index_col=False)
            for label, x, y in zip(top_by_gene.columns, top_gene_trans[:, 0], top_gene_trans[:, 1]):
                if label in genes_plot['GeneID'].tolist():
                    if '_' in label:
                        label = label.split('_')[0]
                    ax_gene.annotate(label, (x+0.1, y+0.1))
        else:
            for label, x, y in zip(top_by_gene.columns, top_gene_trans[:, 0], top_gene_trans[:, 1]):
                if '_' in label:
                    label = label.split('_')[0]
                ax_gene.annotate(label, (x+0.1, y+0.1))
        if title != '':
            save_name = '_'.join(title.split(' ')[0:2])
            plt.savefig(os.path.join(matrix_data.new_filepath,save_name+'_TruncatedSVD.pdf'), bbox_inches='tight')
            #plot_url = py.plot_mpl(fig)
        else:
            #plot_url = py.plot_mpl(fig)
            plt.savefig(os.path.join(matrix_data.new_filepath,'Group0_TruncatedSVD.pdf'), bbox_inches='tight')
        plt.close('all')
        return top_pca_list
    else:
        return []

#create cell and gene TSNE scatter plots (one pdf)
def plot_TSNE(args, matrix_data, title= ''):
    sns.set(context= 'poster', font_scale = 1.2)
    sns.set_palette("RdBu_r", 10, 1)
    if isinstance(matrix_data.log2_df_cell_gene_restricted,pd.DataFrame):
        gene_list = matrix_data.short_gene_list
        len_gene_list = len(gene_list)
        num_genes = min(args.gene_number, len_gene_list)

        df_by_gene = matrix_data.log2_df_cell_gene_restricted.transpose()
    else:
        gene_list = matrix_data.gene_list
        len_gene_list = len(gene_list)
        num_genes = min(args.gene_number, len_gene_list)

        df_by_gene = matrix_data.log2_df_cell.transpose()

    gene_pca = TruncatedSVD(n_components=3)
    np_by_gene = np.asarray(df_by_gene)

    by_gene_trans = gene_pca.fit_transform(np_by_gene)
    Pc_df = pd.DataFrame(gene_pca.components_.T, columns=['PC-1', 'PC-2', 'PC-3'], index=gene_list)
    pca_rank_df = Pc_df.abs().sum(axis=1)
    Pc_sort_df = pca_rank_df.nlargest(len_gene_list)
    top_pca_list = Pc_sort_df.index.tolist()

    top_by_gene = df_by_gene[top_pca_list[0:num_genes]]
    gene_top = TSNE(n_components=2, init='pca', random_state=0)
    cell_pca = TSNE(n_components=2, init='pca', random_state=0)
    top_by_cell = top_by_gene.transpose()
    np_top_gene = np.asarray(top_by_cell)
    np_top_cell = np.asarray(top_by_gene)
    top_cell_trans = cell_pca.fit_transform(np_top_cell)
    top_gene_trans = gene_top.fit_transform(np_top_gene)
    if not np.isnan(top_cell_trans).any():
        fig, (ax_cell, ax_gene) = plt.subplots(2, 1, figsize=(15, 30), sharex=False)
        rect_cell = ax_cell.patch
        rect_gene = ax_gene.patch
        rect_cell.set_facecolor('white')
        rect_gene.set_facecolor('white')
        ax_cell.grid(b=True, which='major', color='grey', linestyle='--', linewidth=0.3)
        ax_gene.grid(b=True, which='major', color='grey', linestyle='--', linewidth=0.3)
        if matrix_data.cell_label_map:
            annotate = args.annotate_cell_pca
            X = [x for x in top_cell_trans[:, 0]]
            Y = [y for y in top_cell_trans[:, 1]]
            for group_index , group_name in enumerate(matrix_data.cell_group_names):
                if group_index == 0:
                    labels = [matrix_data.cell_label_map[cell][group_index][2] for cell in top_by_cell.columns.tolist()]
                    colors = [matrix_data.cell_label_map[cell][group_index][0] for cell in top_by_cell.columns.tolist()]
                elif group_index > 0:
                    labels = [a+'_'+b for a, b in zip(labels, [matrix_data.cell_label_map[cell][group_index][2] for cell in top_by_cell.columns.tolist()])]

            markers = [matrix_data.cell_label_map[cell][0][1] for cell in top_by_cell.columns.tolist()]

            label_done = []
            xy_by_color_dict = {}
            for c in set(colors):
                xy_by_color_dict[c] = []
            for X_pos, Y_pos, m, color, l in zip(X, Y, markers, colors, labels):
                if l in label_done:
                    lab = ''
                else:
                    lab= l
                    label_done.append(l)
                xy_by_color_dict[color].append([X_pos,Y_pos])
                ax_cell.scatter(X_pos, Y_pos, marker=m, c=color, label=lab, s=30)
            if args.add_ellipse:
                for c in set(colors):
                    ell, edge = ellip_enclose(np.asarray(xy_by_color_dict[c]), c)
                    ax_cell.add_artist(ell)
                    ax_cell.add_artist(edge)
        else:
            ax_cell.scatter(top_cell_trans[:, 0], top_cell_trans[:, 1], alpha=0.75)
            annotate = args.annotate_cell_pca
        ax_cell.set_xlim([min(top_cell_trans[:, 0])-1, max(top_cell_trans[:, 0]+1)])
        ax_cell.set_ylim([min(top_cell_trans[:, 1])-1, max(top_cell_trans[:, 1]+2)])
        ax_cell.set_title(title+'_cell')
        if matrix_data.cell_label_map:
            handles, labs = ax_cell.get_legend_handles_labels()
            # sort both labels and handles by labels
            labs, handles = zip(*sorted(zip(labs, handles), key=lambda t: t[0]))
            ax_cell.legend(handles, labs, loc='best', ncol=1, prop={'size':12}, markerscale=1.5, frameon=True)
        ax_cell.set_xlabel('PC1')
        ax_cell.set_ylabel('PC2')
        if annotate:
            for label, x, y in zip(top_by_cell.columns, top_cell_trans[:, 0], top_cell_trans[:, 1]):
                ax_cell.annotate(label, (x+0.1, y+0.1))

        if matrix_data.gene_label_map:
            X = [x for x in top_gene_trans[:, 0]]
            Y = [y for y in top_gene_trans[:, 1]]
            labels = top_by_gene.columns.tolist()
            markers = [matrix_data.gene_label_map[gene][1] for gene in top_by_gene.columns.tolist()]
            colors = [matrix_data.gene_label_map[gene][0] for gene in top_by_gene.columns.tolist()]
            xy_by_color_dict = {}
            for c in set(colors):
                xy_by_color_dict[c] = []
            for X_pos, Y_pos, m, color, l in zip(X, Y, markers, colors, labels):
                xy_by_color_dict[color].append([X_pos,Y_pos])
                ax_gene.scatter(X_pos, Y_pos, marker=m, c=color, label = l, s=30)
            if args.add_ellipse:
                for c in set(colors):
                    ell, edge = ellip_enclose(np.asarray(xy_by_color_dict[c]), c)
                    ax_gene.add_artist(ell)
                    ax_gene.add_artist(edge)

        else:
            ax_gene.scatter(top_gene_trans[:, 0], top_gene_trans[:, 1], alpha=0.75)
        ax_gene.set_xlim([min(top_gene_trans[:, 0])-1, max(top_gene_trans[:, 0])+1])
        ax_gene.set_ylim([min(top_gene_trans[:, 1])-1, max(top_gene_trans[:, 1])+2])
        ax_gene.set_title(title+'_gene')
        ax_gene.set_xlabel('PC1')
        ax_gene.set_ylabel('PC2')
        if args.annotate_gene_subset:
            plot_subset_path = os.path.join(os.path.dirname(args.filepath),args.annotate_gene_subset)
            genes_plot = pd.read_csv(plot_subset_path, sep='\t', index_col=False)
            for label, x, y in zip(top_by_gene.columns, top_gene_trans[:, 0], top_gene_trans[:, 1]):
                if label in genes_plot['GeneID'].tolist():
                    if '_' in label:
                        label = label.split('_')[0]
                    ax_gene.annotate(label, (x+0.1, y+0.1))
        else:
            for label, x, y in zip(top_by_gene.columns, top_gene_trans[:, 0], top_gene_trans[:, 1]):
                if '_' in label:
                    label = label.split('_')[0]
                ax_gene.annotate(label, (x+0.1, y+0.1))

        if title != '':
            save_name = '_'.join(title.split(' ')[0:2])
            plt.savefig(os.path.join(matrix_data.new_filepath,save_name+'_TSNE.pdf'), bbox_inches='tight')
        else:
            plt.savefig(os.path.join(matrix_data.new_filepath,'Group0_TSNE.pdf'), bbox_inches='tight')
        plt.close('all')
        return top_pca_list
    else:
        return []

#create cell and gene TSNE scatter plots (one pdf)
def plot_kmeans(args, matrix_data, kmeans_range, title=''):
    sns.set(context= 'poster', font_scale = 1.2)
    from sklearn.metrics import silhouette_samples, silhouette_score
    try:
        from .heatmaps import clust_heatmap
        from .significance_testing import multi_group_sig
    except SystemError:
        from heatmaps import clust_heatmap
        from significance_testing import multi_group_sig
    import matplotlib.cm as cm
    sns.set_palette("RdBu_r", 10, 1)
    path_filename = matrix_data.new_filepath
    if isinstance(matrix_data.log2_df_cell_gene_restricted,pd.DataFrame):
        gene_list = matrix_data.short_gene_list
        len_gene_list = len(gene_list)
        num_genes = min(args.gene_number, len_gene_list)
        df_by_gene = matrix_data.log2_df_cell_gene_restricted.transpose()

    else:
        gene_list = matrix_data.gene_list
        len_gene_list = len(gene_list)
        num_genes = min(args.gene_number, len_gene_list)
        df_by_gene = matrix_data.log2_df_cell.transpose()


    gene_pca = TruncatedSVD(n_components=3)
    np_by_gene = np.asarray(df_by_gene)

    by_gene_trans = gene_pca.fit_transform(np_by_gene)
    Pc_df = pd.DataFrame(gene_pca.components_.T, columns=['PC-1', 'PC-2', 'PC-3'], index=gene_list)
    pca_rank_df = Pc_df.abs().sum(axis=1)
    Pc_sort_df = pca_rank_df.nlargest(len_gene_list)
    top_pca_list = Pc_sort_df.index.tolist()

    top_by_gene = df_by_gene[top_pca_list[0:num_genes]]
    use_TSNE = False
    if use_TSNE:
        gene_top = TSNE(n_components=2, init='pca', random_state=0)
        cell_pca = TSNE(n_components=2, init='pca', random_state=0)
    else:
        gene_top = TruncatedSVD(n_components=2)
        cell_pca = TruncatedSVD(n_components=2)
    top_by_cell = top_by_gene.transpose()
    np_top_gene = np.asarray(top_by_cell)
    np_top_cell = np.asarray(top_by_gene)
    top_cell_trans = cell_pca.fit_transform(np_top_cell)
    top_gene_trans = gene_top.fit_transform(np_top_gene)

    range_n_clusters = range(kmeans_range[0],kmeans_range[1]+1)

    for n_clusters in range_n_clusters:
        # Create a subplot with 1 row and 2 columns
        fig, (ax1, ax2) = plt.subplots(1, 2)
        fig.set_size_inches(18, 7)

        ax1.set_xlim([-0.1, 1])
        # The (n_clusters+1)*10 is for inserting blank space between silhouette
        # plots of individual clusters, to demarcate them clearly.
        ax1.set_ylim([0, len(np_top_cell) + (n_clusters + 1) * 10])
        #cluster cell PCA
        cell_clusterer = KMeans(n_clusters=n_clusters)
        top_cell_pred = cell_clusterer.fit_predict(top_cell_trans)
        #cluster gene PCA
        gene_clusterer = KMeans(n_clusters=n_clusters)
        top_gene_pred = gene_clusterer.fit_predict(top_gene_trans)

        pred_dict = {'SampleID':top_by_cell.columns, 'GroupID':['kmeans_'+str(p) for p in top_cell_pred]}
        df_pred = pd.DataFrame(pred_dict)
        cell_group_path = os.path.join(path_filename,'kmeans_cell_groups_'+str(n_clusters)+'.txt')
        df_pred.to_csv(cell_group_path, sep = '\t')
        #compute silouette averages and values
        silhouette_avg_cell = silhouette_score(top_cell_trans, top_cell_pred)
        if args.verbose:
            print("For n_clusters =", n_clusters,
              "The average silhouette_score is :", silhouette_avg_cell)
        silhouette_avg_gene = silhouette_score(top_gene_trans, top_gene_pred)


        sample_silhouette_values_cell = silhouette_samples(top_cell_trans, top_cell_pred)
        sample_silhouette_values_gene = silhouette_samples(top_gene_trans, top_gene_pred)

        y_lower = 10
        for i in range(n_clusters):
            # Aggregate the silhouette scores for samples belonging to
            # cluster i, and sort them
            ith_cluster_silhouette_values = \
                sample_silhouette_values_cell[top_cell_pred == i]

            ith_cluster_silhouette_values.sort()

            size_cluster_i = ith_cluster_silhouette_values.shape[0]
            y_upper = y_lower + size_cluster_i

            color = cm.spectral(float(i) / n_clusters)
            ax1.fill_betweenx(np.arange(y_lower, y_upper),
                              0, ith_cluster_silhouette_values,
                              facecolor=color, edgecolor=color, alpha=0.7)

            # Label the silhouette plots with their cluster numbers at the middle
            ax1.text(-0.05, y_lower + 0.5 * size_cluster_i, str(i))

            # Compute the new y_lower for next plot
            y_lower = y_upper + 10  # 10 for the 0 samples

        ax1.set_title("The silhouette plot for the various clusters.")
        ax1.set_xlabel("The silhouette coefficient values")
        ax1.set_ylabel("Cluster label")

        # The vertical line for average silhoutte score of all the values
        ax1.axvline(x=silhouette_avg_cell, color="red", linestyle="--")

        ax1.set_yticks([])  # Clear the yaxis labels / ticks
        ax1.set_xticks([-0.1, 0, 0.2, 0.4, 0.6, 0.8, 1])

        # 2nd Plot showing the actual clusters formed
        colors = cm.spectral(top_cell_pred.astype(float) / n_clusters)
        ax2.scatter(top_cell_trans[:, 0], top_cell_trans[:, 1], marker='.', s=30, lw=0, alpha=0.7,
                    c=colors)

        # Labeling the clusters
        centers = cell_clusterer.cluster_centers_
        # Draw white circles at cluster centers
        ax2.scatter(centers[:, 0], centers[:, 1],
                    marker='o', c="white", alpha=1, s=200)

        for i, c in enumerate(centers):
            ax2.scatter(c[0], c[1], marker='$%d$' % i, alpha=1, s=50)

        ax2.set_title("The visualization of the clustered data.")
        ax2.set_xlabel("Feature space for the 1st feature")
        ax2.set_ylabel("Feature space for the 2nd feature")

        plt.suptitle(("Silhouette analysis for KMeans clustering on sample data "
                      "with n_clusters = %d" % n_clusters),
                     fontsize=14, fontweight='bold')

        plt.savefig(os.path.join(path_filename,'Group0_kmeans_'+str(n_clusters)+'_clusters.pdf'), bbox_inches='tight')
        plt.close('all')

        #use colors to make label map compatable with heatmap
        color_dict ={}
        for cell, m_color_tup in zip(top_by_cell.columns, zip(colors,[matrix_data.markers[pred] for pred in top_cell_pred],['kmeans_'+str(p) for p in top_cell_pred])):
            color_dict[cell] = []
            color_dict[cell].append(m_color_tup)

        group_color_dict = dict(zip(['kmeans_'+str(p) for p in top_cell_pred],zip(colors,[matrix_data.markers[pred] for pred in top_cell_pred])))
        #run heatmap with kmeans clustering and colors

        top_pca_by_gene, top_pca = return_top_pca_gene(args, df_by_gene.transpose())
        cell_linkage, plotted_df_by_gene, col_order = clust_heatmap(args, matrix_data, matrix_subset=top_pca_by_gene, title= 'kmeans_label_with_'+str(n_clusters)+'_clusters', kmeans_color_map=color_dict)
        if args.kmeans_sig_test:
            sns.set(context= 'poster', font_scale = 1.2-(n_clusters*.08))
            multi_group_sig(args, matrix_data, from_kmeans='kmeans_'+str(n_clusters), alt_color_dict=group_color_dict, kmeans_groups_file=cell_group_path)



import numpy as np
import pandas as pd
import os
import sys
import matplotlib
matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
import scipy
import json

from scipy.cluster import hierarchy
import seaborn as sns
from functools import reduce
import matplotlib.patches as patches
import warnings





# Create a nested dictionary from the ClusterNode's returned by SciPy
def add_node(node, parent):
	# First create the new node and append it to its parent's children
	newNode = dict( node_id=node.id, children=[] )
	parent["children"].append( newNode )

	# Recursively add the current node's children
	if node.left: add_node( node.left, newNode )
	if node.right: add_node( node.right, newNode )


cc = []
# Label each node with the names of each leaf in its subtree
def label_tree(n, id2name):
    # If the node is a leaf, then we have its name
    if len(n["children"]) == 0:
        leafNames = [ id2name[n["node_id"]] ]

    # If not, flatten all the leaves in the node's subtree
    else:
        leafNames = reduce(lambda ls, c: ls + label_tree(c,id2name), n["children"], [])

    cc.append((len(leafNames), [x.strip('\n') for x in leafNames]))
    cc.sort(key=lambda tup: tup[0], reverse = True)

    # Delete the node id since we don't need it anymore and
    # it makes for cleaner JSON
    del n["node_id"]

    # Labeling convention: "-"-separated leaf names
    n["name"] = name = "-".join(sorted(map(str, leafNames)))

    return leafNames

#Makes labeled json tree for visulaization in d3, makes and returns cc object within label_tree
def make_tree_json(row_clusters, df_by_gene, path_filename):
    T= hierarchy.to_tree(row_clusters)

    # Create dictionary for labeling nodes by their IDs
    labels = list(df_by_gene.index)
    id2name = dict(zip(range(len(labels)), labels))

    # Initialize nested dictionary for d3, then recursively iterate through tree
    d3Dendro = dict(children=[], name="Root1")
    add_node( T, d3Dendro )
    label_tree( d3Dendro["children"][0], id2name )
    # Output to JSON
    json.dump(d3Dendro, open(os.path.join(path_filename,"d3-dendrogram.json"), "w"), sort_keys=True, indent=4)

    return cc


#finds significant genes between subclusters
def find_twobytwo(cc, args, matrix_data):
    if not args.verbose:
        warnings.filterwarnings("ignore", category=RuntimeWarning)
    gene_list = matrix_data.gene_list
    by_gene_df = matrix_data.log2_df_cell.transpose()
    pair_dict = {}
    parent = cc[0][1]
    p_num = cc[0][0]
    l_nums = [x[0] for x in cc]
    c_lists = [c[1] for c in cc[1:]]
    unique_count = 1
    pair_list = []
    for i, c in enumerate(c_lists):
        for i2, c2 in enumerate(c_lists):
            overlap = [i for i in c if i in c2]
            if not overlap and len(c)>=args.depth_of_clustering and len(c2)>=args.depth_of_clustering:
                if (c,c2) not in pair_list:
                    pair_list.append((c,c2))
                    pair_list.append((c2,c))
                    pair_dict[str(len(c))+'cells_vs_'+str(len(c2))+'cells'+str(unique_count)]= [c, c2]
                    unique_count+=1

    for v, k in pair_dict.items():
        g_pvalue_dict = {}
        index_list = []
        sig_gene_list = []
        cell_list1 = [x.strip('\n') for x in k[0]]
        cell_list2 = [xx.strip('\n') for xx in k[1]]
        group1 = str(len(cell_list1))
        group2 = str(len(cell_list2))
        df_by_cell_1 = matrix_data.log2_df_cell[cell_list1]
        df_by_cell_2 = matrix_data.log2_df_cell[cell_list2]
        df_by_gene_1 = df_by_cell_1.transpose()
        df_by_gene_2 = df_by_cell_2.transpose()
        for g in gene_list:
            g_pvalue = scipy.stats.f_oneway(df_by_gene_1[g], df_by_gene_2[g])
            if g_pvalue[0] > 0 and g_pvalue[1] <= 1:
                g_pvalue_dict[g] = g_pvalue
                if g not in [s[0] for s in sig_gene_list]:
                    sig_gene_list.append([g, g_pvalue[1]])

        sig_gene_list.sort(key=lambda tup: tup[1])
        pvalues = [p[1] for p in sig_gene_list]
        gene_index = [ge[0] for ge in sig_gene_list]
        mean_log2_exp_list = []
        sig_1_2_list = []
        mean1_list = []
        mean2_list = []
        for sig_gene in gene_index:
            sig_gene_df = by_gene_df[sig_gene]
            mean_log2_exp_list.append(sig_gene_df.mean())
            sig_cell_df = sig_gene_df.transpose()
            mean_cell1 = sig_cell_df[cell_list1].mean()
            mean1_list.append(mean_cell1)
            mean_cell2 = sig_cell_df[cell_list2].mean()
            mean2_list.append(mean_cell2)
            ratio_1_2 = (mean_cell1+1)/(mean_cell2+1)
            sig_1_2_list.append(ratio_1_2)
        sig_df = pd.DataFrame({'pvalues':pvalues,'mean_all':mean_log2_exp_list,'mean_group1':mean1_list, 'mean_group2':mean2_list, 'ratio_1_2':sig_1_2_list}, index=gene_index)
        cell_names_df = pd.DataFrame({'cells1':pd.Series(cell_list1, index=range(len(cell_list1))), 'cells2':pd.Series(cell_list2, index=range(len(cell_list2)))})
        sig_df.to_csv(os.path.join(matrix_data.new_filepath,'sig_'+v+'_pvalues.txt'), sep = '\t')
        cell_names_df.to_csv(os.path.join(matrix_data.new_filepath,'sig_'+v+'_cells.txt'), sep = '\t')







def clust_heatmap(args, matrix_data, title= '', matrix_subset = None, fontsize=18, stablity_plot_num = 0, kmeans_color_map= {}, plot=True):
    try:
        from .dim_reduction import return_top_pca_gene
    except SystemError:
        from dim_reduction import return_top_pca_gene
    if isinstance(matrix_subset,pd.DataFrame):
        cell_list = matrix_subset.index.tolist()
        cell_num =len(cell_list)
        df_by_gene = matrix_subset

        num_to_plot = min(args.gene_number, len(matrix_subset.columns.tolist()))
    else:
        if isinstance(matrix_data.log2_df_cell_gene_restricted,pd.DataFrame):
            cell_list = matrix_data.cell_list
            cell_num =len(cell_list)
            df_by_gene = matrix_data.log2_df_cell_gene_restricted.transpose()
            num_to_plot = min(args.gene_number, len(matrix_data.short_gene_list))
        else:
            cell_list = matrix_data.cell_list
            cell_num =len(cell_list)
            df_by_gene = matrix_data.log2_df_cell.transpose()
            num_to_plot = min(args.gene_number, len(matrix_data.gene_list))

    if kmeans_color_map:
        cell_color_map = kmeans_color_map
        first_group_name = 'Kmeans_groups'
        groups_list = ['Kmeans_groups']
    elif matrix_data.cell_label_map:
        cell_color_map = matrix_data.cell_label_map
        first_group_name = matrix_data.cell_group_names[0]
        groups_list = matrix_data.cell_group_names
    else:
        cell_color_map = {}

    if stablity_plot_num != 0:
        num_to_plot = stablity_plot_num
    top_pca_by_gene, top_pca_gene_list = return_top_pca_gene(args, df_by_gene.transpose(), user_num_genes=num_to_plot)
    longest_side = max(num_to_plot,cell_num*2)
    if longest_side > 500:
        fontsize=10
    elif longest_side >250:
        fontsize=14
    if longest_side == num_to_plot:
        sns.set(context= 'poster', font_scale = min(.4*(num_to_plot/100),3))
        width_heatmap = min(28+round(cell_num/50),42+round(cell_num/40))
        len_heatmap = min(43+round(num_to_plot/8),68+round(num_to_plot/20))
        title_set = 1.15
    else:
        sns.set(context= 'poster', font_scale = .6*(cell_num/120))
        width_heatmap = min(42+round(cell_num/9),68+round(cell_num/40))
        len_heatmap = min(47+round(num_to_plot/8),50+round(num_to_plot/30))
        title_set = 1.12
    font = {'size'   : fontsize}

    plt.rc('font', **font)
    if len(str(args.z_direction)) > 1:
        z_choice = str(args.z_direction)
        if z_choice != 'None':
            sys.exit('Please enter a valid option (0, 1, or None) for z_direction')
    else:
        z_choice = int(args.z_direction)
        if z_choice != 0 and z_choice != 1:
            sys.exit('Please enter a valid option (0, 1, or None) for z_direction')
    cmap = sns.diverging_palette(255, 10, s=99, sep=1, as_cmap=True)
    gene_list = top_pca_gene_list[0:num_to_plot]
    cluster_df = df_by_gene[gene_list].transpose()
    cluster_df[abs(cluster_df)<3e-12] = 0.0
    try:
        cg = sns.clustermap(cluster_df, method=args.method, metric=args.metric, z_score=z_choice, figsize=(width_heatmap, len_heatmap), cmap =cmap)
        col_order = cg.dendrogram_col.reordered_ind
        row_order = cg.dendrogram_row.reordered_ind

        if cell_color_map and matrix_data.gene_label_map and stablity_plot_num == 0:
            Xlabs = [cell_list[i] for i in col_order]
            Xcolors = {}
            for group_index , group_name in enumerate(groups_list):
                Xcolors[group_name]=[cell_color_map[cell][group_index][0] for cell in Xlabs]
            col_colors = pd.DataFrame(Xcolors,index=Xlabs)
            Xgroup_labels = [cell_color_map[cell][0][2] for cell in Xlabs]
            Ylabs = [gene_list[i] for i in row_order]
            Ycolors = [matrix_data.gene_label_map[gene][0] for gene in Ylabs]
            Ygroup_labels= [matrix_data.gene_label_map[gene][2] for gene in Ylabs]
            row_colors = pd.DataFrame({'Gene Groups': Ycolors},index=Ylabs)
            cg = sns.clustermap(cluster_df, method=args.method, metric=args.metric, z_score=z_choice, row_colors=row_colors, col_colors=col_colors, figsize=(width_heatmap, len_heatmap), cmap =cmap)
        elif cell_color_map:
            Xlabs = [cell_list[i] for i in col_order]
            Xcolors = {}
            for group_index , group_name in enumerate(groups_list):
                Xcolors[group_name]=[cell_color_map[cell][group_index][0] for cell in Xlabs]
            col_colors = pd.DataFrame(Xcolors,index=Xlabs)
            Xgroup_labels = [cell_color_map[cell][0][2] for cell in Xlabs]
            cg = sns.clustermap(cluster_df, method=args.method, metric=args.metric, z_score=z_choice, col_colors=col_colors, figsize=(width_heatmap, len_heatmap), cmap =cmap)
        elif matrix_data.gene_label_map and stablity_plot_num == 0:
            Ylabs = [gene_list[i] for i in row_order]
            Ycolors = [matrix_data.gene_label_map[gene][0] for gene in Ylabs]
            Ygroup_labels= [matrix_data.gene_label_map[gene][2] for gene in Ylabs]
            row_colors = pd.DataFrame({'Gene Groups': Ycolors},index=Ylabs)
            cg = sns.clustermap(cluster_df, method=args.method, metric=args.metric, z_score=z_choice,row_colors=row_colors, figsize=(width_heatmap, len_heatmap), cmap =cmap)

        cg.ax_heatmap.set_title(title, y=title_set)
        cg.cax.set_title('Z-score')
        if cell_color_map:

            for xtick, xcolor, xgroup_name in zip(cg.ax_heatmap.get_xticklabels(), [cell_color_map[cell][0][0] for cell in Xlabs], Xgroup_labels):
                xtick.set_color(xcolor)
                xtick.set_rotation(270)
                xtick.set_fontsize(fontsize)

            leg_handles_cell = {}

            for group_index , group_name in enumerate(groups_list):
                group_seen_cell = []
                leg_handles_cell[group_name] = []
                for xcolor, xgroup_name in zip([cell_color_map[cell][group_index][0] for cell in Xlabs], [cell_color_map[cell][group_index][2] for cell in Xlabs]):
                    if xgroup_name not in group_seen_cell:
                        leg_handles_cell[group_name].append(patches.Patch(color=xcolor, label=xgroup_name))
                        group_seen_cell.append(xgroup_name)
        else:
            for xtick in cg.ax_heatmap.get_xticklabels():
                xtick.set_rotation(270)
                xtick.set_fontsize(fontsize)
        if matrix_data.gene_label_map and stablity_plot_num == 0:
            leg_handles_gene =[]
            group_seen_gene = []
            for ytick, ycolor, ygroup_name in zip(cg.ax_heatmap.get_yticklabels(), list(reversed(Ycolors)), list(reversed(Ygroup_labels))):
                ytick.set_color(ycolor)
                ytick.set_rotation(0)
                ytick.set_fontsize(fontsize)
                if ygroup_name not in group_seen_gene:
                    leg_handles_gene.append(patches.Patch(color=ycolor, label=ygroup_name))
                    group_seen_gene.append(ygroup_name)
        else:
            for ytick in cg.ax_heatmap.get_yticklabels():
                ytick.set_rotation(0)
                ytick.set_fontsize(fontsize)
        if matrix_data.gene_label_map and cell_color_map and stablity_plot_num == 0:
            gene_legend = cg.ax_heatmap.legend(handles=leg_handles_gene, loc=2, bbox_to_anchor=(1.04, 1-(0.2*len(groups_list))), title='Gene groups', prop={'size':fontsize})
            plt.setp(gene_legend.get_title(),fontsize=fontsize)
            cg.ax_heatmap.add_artist(gene_legend)
            for group_count ,group_name in enumerate(groups_list):
                cell_legend = cg.ax_heatmap.legend(handles=leg_handles_cell[group_name], loc=2, bbox_to_anchor=(1.04, 1-(0.2*group_count)), title='Cell groups '+group_name, prop={'size':fontsize})
                if group_count == 0:
                    cg.ax_heatmap.add_artist(cell_legend)
                else:
                    plt.setp(cell_legend.get_title(),fontsize=fontsize)
                #cg.ax_heatmap.add_artist(cell_legend)
        elif cell_color_map:
            for group_count ,group_name in enumerate(groups_list):
                cell_legend = cg.ax_heatmap.legend(handles=leg_handles_cell[group_name], loc=2, bbox_to_anchor=(1.04, 1-(0.2*group_count)), title='Cell groups '+group_name, prop={'size':fontsize})
                if group_count == 0:
                    cg.ax_heatmap.add_artist(cell_legend)
                else:
                    plt.setp(cell_legend.get_title(),fontsize=fontsize)

        elif matrix_data.gene_label_map and stablity_plot_num == 0:
            gene_legend = cg.ax_heatmap.legend(handles=leg_handles_gene, loc=2, bbox_to_anchor=(1.04, 0.8), title='Gene groups', prop={'size':fontsize})
            plt.setp(gene_legend.get_title(),fontsize=fontsize)

        cell_linkage = cg.dendrogram_col.linkage

        link_mat = pd.DataFrame(cell_linkage,
                    columns=['row label 1', 'row label 2', 'distance', 'no. of items in clust.'],
                    index=['cluster %d' %(i+1) for i in range(cell_linkage.shape[0])])
        if plot:
            if title != '':
                save_name = '_'.join(title.split(' ')[0:2])
                #plot_url = py.plot_mpl(cg)
                cg.savefig(os.path.join(matrix_data.new_filepath, save_name+'_heatmap.pdf'), bbox_inches='tight')
            else:
                #plot_url = py.plot_mpl(cg)
                cg.savefig(os.path.join(matrix_data.new_filepath,'Group0_Heatmap_all_cells.pdf'), bbox_inches='tight')
            plt.close('all')
        return cell_linkage, df_by_gene[gene_list], col_order
    except FloatingPointError:
        print('Linkage distance has too many zeros. Filter to remove non-expressed genes in order to produce heatmap. Heatmap with '+ str(len(cell_list))+' will not be created.')
        return False, False, False

def make_subclusters(args, cc, matrix_data):
    '''
    Walks a histogram branch map 'cc' and does PCA (SVD), heatmap and correlation search for each non-overlapping
    tree branch. Stops at defined cluster_size (default is 20).
    '''
    try:
        from .dim_reduction import return_top_pca_gene, plot_SVD
    except SystemError:
        from dim_reduction import return_top_pca_gene, plot_SVD

    #initial cell group is parent
    parent = cc[0][1]
    #p_num is the number of cells in the parent group
    p_num = cc[0][0]
    #l_nums is number of members of each leaf of tree
    l_nums = [x[0] for x in cc]
    #cell list is the list of list of cells in each leaf of tree
    c_lists = [c[1] for c in cc]
    #Group ID will increment with each group so that each subcluter has a unique ID
    group_ID = 0

    for num_members, cell_list in zip(l_nums, c_lists):
        #run all cell groups that are subgroups of the parent and greater than or equal to the cutoff args.depth_of_clustering
        if num_members < p_num and num_members >= args.depth_of_clustering:
            group_ID+=1
            #save name for all files generated within this cluster i.e. 'Group_2_with_105_cells_heatmap.pdf'
            current_title = 'Group_'+str(group_ID)+'_with_'+str(num_members)+'_cells'

            if isinstance(matrix_data.log2_df_cell_gene_restricted,pd.DataFrame):
                cell_subset = matrix_data.log2_df_cell_gene_restricted.loc[:,list(set(cell_list))]
                gene_subset = cell_subset.transpose()
                gene_subset = gene_subset.loc[:,(gene_subset!=0).any()]

                full_cell_subset = matrix_data.log2_df_cell.loc[:,list(set(cell_list))]
                full_gene_subset = full_cell_subset.transpose()
                full_gene_subset = full_gene_subset.loc[:,(full_gene_subset!=0).any()]
                num_genes = min(args.gene_number, len(matrix_data.short_gene_list))
                top_pca_by_gene, top_pca = return_top_pca_gene(args, cell_subset)
                plot_SVD(args, matrix_data, gene_subcluster_matrix=gene_subset, title=current_title)
            else:
                full_cell_subset = matrix_data.log2_df_cell.loc[:,list(set(cell_list))]
                full_gene_subset = full_cell_subset.transpose()
                full_gene_subset = full_gene_subset.loc[:,(full_gene_subset!=0).any()]
                top_pca_by_gene, top_pca = return_top_pca_gene(args, full_cell_subset)
                plot_SVD(args,matrix_data, gene_subcluster_matrix=full_gene_subset, title=current_title)

            norm_df_cell1 = np.exp2(full_cell_subset)
            norm_df_cell = norm_df_cell1 -1
            norm_df_cell.to_csv(os.path.join(matrix_data.new_filepath, args.base_name+'_'+current_title+'_matrix.txt'), sep = '\t', index_label=0)

            if len(top_pca)<args.gene_number:
                plot_num = len(top_pca)
            else:
                plot_num = args.gene_number

            if top_pca != []:
                top_pca_by_cell = top_pca_by_gene.transpose()
                #if no_corr flag is provided (False) no correlation plots will be made
                if not args.no_corr:
                    try:
                        from .correlation import corr_plot
                    except SystemError:
                        from correlation import corr_plot
                    top_genes_search = top_pca[0:50]
                    corr_plot(top_genes_search, full_gene_subset, args, matrix_data, title = current_title)

                    cell_linkage, plotted_df_by_gene, col_order = clust_heatmap(args, matrix_data, matrix_subset = top_pca_by_gene, title=current_title)
                plt.close('all')
            else:
                print('Search for top genes by PCA failed in '+current_title+'. No plots will be generated for this subcluster. ')
                pass




import numpy as np
import pandas as pd
import os
import sys
import matplotlib
matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
import seaborn as sns
import math
import collections






#Takes raw matrix data and returns matrix object
class Matrix_filter(object):

    def __init__(self, cell_df, args, cell_list_filepath, gene_list_filepath):

        self.cell_list_filepath = cell_list_filepath
        self.gene_list_filepath = gene_list_filepath
        self.cell_names = cell_df.columns.tolist()
        self.gene_names = cell_df.index.tolist()
        self.data_by_cell = cell_df
        if args.base_name:
            self.new_filepath = os.path.join(os.path.dirname(args.filepath),args.base_name+'_scicast_analysis')
        else:
            self.new_filepath = os.path.join(os.path.dirname(args.filepath),'scicast_analysis')


        self.markers = ['o', 'v','D','*','x','h', 's','p','8','^','>','<', 'd','o', 'v','D','*','x','h', 's','p','8','^','>','<', 'd']
        from matplotlib import colors
        all_color_list = list(colors.cnames.keys())
        self.cell_color_list = ['b', 'm', 'r', 'c', 'g','y','k']+all_color_list
        colors_to_remove = ['gray', 'white', 'oldlace', 'silver', 'lightgrey', 'grey', 'linen', 'snow', 'dimgray', 'slategray', 'dimgrey', 'lightslategrey', 'antiquewhite', 'beige']
        self.cell_color_list = [color for color in self.cell_color_list if color not in colors_to_remove]

        if args.exclude_genes:
            excluded_genes_df = pd.read_csv(os.path.join(os.path.dirname(args.filepath),args.exclude_genes), sep='\t', header=0, index_col=False)
            try:
                self.exclude_list = [g1 for g1 in excluded_genes_df['GeneID'].tolist() if g1 in self.gene_names]
                not_in_mat = [g1 for g1 in excluded_genes_df['GeneID'].tolist() if g1 not in self.gene_names]
            except KeyError:
                sys.exit("Error: Please provide Gene Exclude list file with 'GeneID' as header.")

            self.data_by_cell.drop(self.exclude_list, axis=0, inplace=True)
            if args.verbose:
                print(str(not_in_mat)+' already filtered from matrix.')
        else:
            self.exclude_list = []

        #if cells are to be excluded filter them out and generate log2 matrix otherwise just generate log2_matrix
        if args.limit_cells and self.cell_list_filepath:
            self.make_new_matrix_cell()
        else:
            self.cell_list = self.cell_names
        self.gene_list = self.data_by_cell.index.tolist()

        #if a gene list is provided create the new matrix with only those genes included
        if self.gene_list_filepath:
            self.make_new_matrix_gene(self.gene_list_filepath, v=args.verbose)
        else:
            self.gene_group_list = []
            self.short_gene_list, self.short_gene_matrix_cell =[],[]


        self.log2_outlierfilter(args)

        #if user input color and or marker values are provided generate color_dict
        if args.color_cells:
            self.color_dict_cells= {}
            color_list1 = args.color_cells.split(' ')
            for i, c in enumerate(color_list1):
                c_pair = c.split(',')
                if len(c_pair) == 2:
                    self.color_dict_cells[c_pair[0]] = [c_pair[1],self.markers[i]]
                elif len(c_pair == 3):
                    self.color_dict_cells[c_pair[0]] = [c_pair[1],c_pair[2]]
        #otherwise generate generic color dict from colors and markers
        elif self.cell_list_filepath:
            self.color_dict_cells= {}
            self.cell_group_set = {}
            cell_groups_df = pd.read_csv(open(self.cell_list_filepath,'rU'), sep=None, engine='python')
            self.cell_group_names = cell_groups_df.columns.tolist()[1:]
            cell_group_num = len(self.cell_group_names)
            color_marker_start = 0
            for group_name in self.cell_group_names:
                self.cell_group_set[group_name] = list(set(cell_groups_df[group_name].tolist()))
                group_member_num = len(self.cell_group_set[group_name])+color_marker_start
                for i, group in enumerate(self.cell_group_set[group_name]):
                    try:
                        if math.isnan(float(group)):
                            no_groups = True
                            self.cell_group_set[group_name][i] = 'None'
                    except ValueError:
                        pass
                for g,c,m in zip(self.cell_group_set[group_name], self.cell_color_list[color_marker_start:group_member_num],self.markers[color_marker_start:group_member_num]):
                    if g != 'None':
                        self.color_dict_cells[g] =[c,m]
                    else:
                        self.color_dict_cells[g] =['white',m]
                color_marker_start = group_member_num


        #if gene user color inputs are provided
        if args.color_genes:
            self.color_dict_genes= {}
        #if the gene group names are the same as cell groups
        if args.color_genes =='same':
            self.color_dict_genes = self.color_dict_cells
        elif args.color_genes:
            color_list1 = args.color_genes.split(' ')
            for i, c in enumerate(color_list1):
                c_pair = c.split(',')
                if len(c_pair) == 2:
                    self.color_dict_genes[c_pair[0]] = [c_pair[1],self.markers[i]]
                elif len(c_pair == 3):
                    self.color_dict_genes[c_pair[0]] = [c_pair[1],c_pair[2]]
        elif self.gene_list_filepath:
            no_groups = False
            self.color_dict_genes= {}
            gene_groups_df = pd.read_csv(open(self.gene_list_filepath,'rU'), sep=None, engine='python')

            gene_group_set = list(set(gene_groups_df['GroupID'].tolist()))
            for i, group in enumerate(gene_group_set):
                try:
                    if math.isnan(float(group)):
                        no_groups = True
                        gene_group_set[i] = ' '
                except ValueError:
                    pass
            if not no_groups:
                for g,c,m in zip(gene_group_set, self.cell_color_list[0:len(gene_group_set)],self.markers[0:len(gene_group_set)]):
                    self.color_dict_genes[g] =[c,m]
            else:
                self.color_dict_genes[' '] =['b','o']

        self.cell_color_map(args)
        self.gene_color_map(args)

        self.log2_df_cell.to_csv(os.path.join(self.new_filepath, 'log2_matrix_after_filtering.txt'), sep= '\t', index_label=0)
        self.data_by_cell.to_csv(os.path.join(self.new_filepath, 'count_matrix_after_filtering.txt'), sep= '\t', index_label=0)

    def threshold_genes(self, by_cell, row_sum=1, cell_sum=1):
        return by_cell.loc[by_cell.sum(1) > row_sum, by_cell.sum(0) > cell_sum]

    def index_to_label(self,index):
        """Convert a pandas index or multiindex to an axis label."""
        if isinstance(index, pd.MultiIndex):
            return "-".join(map(str, index.names))
        else:
            return index.name

    def make_new_matrix_gene(self, gene_list_or_file, v = False):
        from itertools import compress
        if isinstance(gene_list_or_file,str):
            gene_df = pd.read_csv(open(gene_list_or_file,'rU'), sep=None, engine='python')
            try:
                self.short_gene_list = list(set(gene_df['GeneID'].tolist()))
                if self.exclude_list:

                    self.short_gene_list = [g for g in self.short_gene_list if g not in self.exclude_list and g in self.gene_list]

            except KeyError:
                sys.exit("Error: Please provide Gene list file with 'GeneID' as header.")
            try:
                if self.exclude_list:
                    mask = [True if g not in self.exclude_list and g in self.gene_list else False for g in self.short_gene_list]
                    self.gene_group_list = list(compress(gene_df['GroupID'].tolist(), mask))
                else:
                    self.gene_group_list = gene_df['GroupID'].tolist()
            except KeyError:
                sys.exit("Error: Please provide Gene list file with 'GroupID' as header.")
            try:
                self.short_gene_matrix_cell = self.threshold_genes(self.data_by_cell.loc[self.short_gene_list,:])

            except KeyError as error_gene:
                cause1 = error_gene.args[0].strip(' not in index')
                cause = [error_v.strip('\n\' ') for error_v in cause1.strip('[]').split(' ')]
                absent_gene = cause
                if args.verbose:
                    print(' '.join(absent_gene)+' not in matrix file.')
                new_list = [x for x in self.gene_list if x not in absent_gene]
                self.short_gene_matrix_cell = self.threshold_genes(self.data_by_cell.loc[new_list,:])

        elif isinstance(gene_list_or_file,list):
            if self.exclude_list == []:
                self.short_gene_list = gene_list_or_file
            else:
                self.short_gene_list = [g for g in gene_list_or_file if g not in self.exclude_list and g in self.gene_list]
            try:
                self.short_gene_matrix_cell = self.threshold_genes(self.data_by_cell.loc[self.short_gene_list,:])
            except KeyError as error_gene:
                cause = error_gene.args[0]
                absent_gene = cause.split('\'')[1]
                if v:
                    print(absent_gene+' not in matrix file.')
                new_list = [x for x in self.gene_list if x not in [absent_gene]]
                self.short_gene_matrix_cell = self.threshold_genes(self.data_by_cell.loc[new_list,:])
        else:
            sys.exit("Error: gene list must be filepath or a list.")

    def make_new_matrix_cell(self):
        cell_df = pd.read_csv(open(self.cell_list_filepath,'rU'), sep=None, engine='python')
        cell_list_new = list(set([cell.strip('\n') for cell in cell_df['SampleID'].tolist()]))
        cell_list_old = self.data_by_cell.columns.tolist()
        overlap = [c for c in cell_list_new if c in cell_list_old]
        not_in_matrix = [c for c in cell_list_new if c not in cell_list_old]
        if not_in_matrix != []:
            print('These cells were in the cell list provided, but not found in the matrix provided:')
            print(not_in_matrix)
        self.data_by_cell = self.threshold_genes(self.data_by_cell.ix[:,overlap])
        self.cell_list = self.data_by_cell.columns.tolist()






    def find_top_common_genes(self):
        top_common_list = []
        count = 0
        done = False
        log2_df_by_gene = self.log2_df_cell.transpose()
        log2_df2_gene = log2_df_by_gene.apply(pd.to_numeric,errors='coerce')
        log_mean = log2_df2_gene.mean(axis=0).sort_values(ascending=False)
        try:
            log2_sorted_gene = log2_df_by_gene.reindex_axis(log2_df_by_gene.mean(axis=0).sort_values(ascending=False).index, axis=1)
        except ValueError:
            overlap_list = [item for item, count in collections.Counter(self.log2_df_cell.index).items() if count > 1]
            print(overlap_list, len(overlap_list))
            sys.exit('Error: Duplicate GeneIDs are present.')
        for gene in log2_sorted_gene.columns.tolist():
            if sum(genes < 1 for genes in log2_df_by_gene[gene])<6:
                if count < 20:
                    count+=1
                    top_common_list.append(gene)
            if count == 20:
                done = True
                break
        if done:
            return log2_df_by_gene[top_common_list].transpose()
        else:
            return [0]

    def log2_outlierfilter(self, args):
        if not args.already_log2:
            self.log2_df_cell = np.log2(self.data_by_cell+1)
            if isinstance(self.short_gene_matrix_cell, pd.DataFrame):
                self.log2_df_cell_gene_restricted = np.log2(self.short_gene_matrix_cell+1)
            else:
                self.log2_df_cell_gene_restricted = []
        else:
            self.log2_df_cell = self.data_by_cell
            if isinstance(self.short_gene_matrix_cell, pd.DataFrame):
                self.log2_df_cell_gene_restricted = self.short_gene_matrix_cell
            else:
                self.log2_df_cell_gene_restricted = []

        top_log2 = self.find_top_common_genes()
        if all(top_log2) != 0:
            log2_df= self.log2_df_cell.apply(pd.to_numeric,errors='coerce')
            log_mean = top_log2.mean(axis=0).sort_values(ascending=False)
            log2_sorted = top_log2.reindex_axis(top_log2.mean(axis=0).sort_values(ascending=False).index, axis=1)
            xticks = []
            keep_col= []
            log2_cutoff = np.average(np.average(log2_sorted))-2*np.average(np.std(log2_sorted))
            for col, m in zip(log2_sorted.columns.tolist(),log2_sorted.mean()):
                if m > log2_cutoff:
                    keep_col.append(col)
                    xticks.append(col+' '+str("%.2f" % m))
            excluded_cells = [x for x in log2_sorted.columns.tolist() if x not in keep_col]

            filtered_df_by_cell = self.data_by_cell.ix[:,keep_col]
            if isinstance(self.short_gene_matrix_cell, pd.DataFrame):
                filtered_df_by_cell_gene_restricted = self.short_gene_matrix_cell.ix[:,keep_col]

            if not args.already_log2:
                filtered_df_by_cell +=1
                filtered_log2 = np.log2(filtered_df_by_cell[filtered_df_by_cell>0])
                if isinstance(self.short_gene_matrix_cell, pd.DataFrame):
                    filtered_df_by_cell_gene_restricted += 1
                    filtered_df_by_cell_gene_restricted_log2 = np.log2(filtered_df_by_cell_gene_restricted[filtered_df_by_cell_gene_restricted>0])

            else:
                filtered_log2 = filtered_df_by_cell[filtered_df_by_cell>0]
                if isinstance(self.short_gene_matrix_cell, pd.DataFrame):
                    filtered_df_by_cell_gene_restricted_log2 = filtered_df_by_cell_gene_restricted[filtered_df_by_cell_gene_restricted>0]
            fig1, ax1 = plt.subplots(figsize=(35,15))
            sns.boxplot(data=filtered_log2, whis= .75, notch=True, ax=ax1)
            sns.stripplot(x=filtered_log2.columns.values, y=filtered_log2.mean(axis=0), size=4, jitter=True, edgecolor="gray", ax=ax1)
            xtickNames = plt.setp(ax1, xticklabels=xticks)
            plt.setp(xtickNames, rotation=90, fontsize=9)
            plt.savefig(os.path.join(self.new_filepath, 'top_log2_genes_plot.pdf'), bbox_inches='tight')
            plt.clf()
            fig2, ax2 = plt.subplots(figsize=(12,10))
            sns.distplot(filtered_log2.mean(), color='r', label='After filtering', ax=ax2)
            sns.distplot(self.log2_df_cell.mean(), color='b', hist=False, label='Before filtering', ax=ax2)
            plt.savefig(os.path.join(self.new_filepath, 'distribution_log2_genes_plot.pdf'), bbox_inches='tight')

            self.log2_df_cell = filtered_log2
            if isinstance(self.short_gene_matrix_cell, pd.DataFrame):
                self.log2_df_cell_gene_restricted = self.threshold_genes(filtered_df_by_cell_gene_restricted_log2)
            else:
                self.log2_df_cell_gene_restricted = []
        else:
            if args.verbose:
                print("no common genes found")

    '''takes cell groups and creates dictionay 'cell_label_map' that has attached color and marker
    if the same cell is assigned to multiple groups it assigns it to the first groupID
    '''
    def cell_color_map(self, args):
        if self.cell_list_filepath:
            cell_groups_df = pd.read_csv(open(os.path.join(os.path.dirname(args.filepath), self.cell_list_filepath),'rU'), sep=None, engine='python')
            cell_list_1 = list(set(self.cell_list))
            self.cell_label_map = {}
            for cell1 in cell_list_1:
                self.cell_label_map[cell1] = []
            for group_name in self.cell_group_names:
                group_seen = []
                cells_seen = []


                for cell, group in list(set(zip(cell_groups_df['SampleID'].tolist(), cell_groups_df[group_name].tolist()))):
                    if cell not in cells_seen:
                        try:
                            if math.isnan(float(group)):
                                group = 'None'
                        except ValueError:
                            pass
                        self.cell_label_map[cell].append((self.color_dict_cells[group][0] , self.color_dict_cells[group][1] , group))
                        cells_seen.append(cell)
                non_group_cells = [c for c in cell_list_1 if c not in cells_seen]
                if non_group_cells != []:
                    for cell in non_group_cells:
                        group = 'None'
                        self.cell_label_map[cell].append(('w','8',group))
        else:
            self.cell_label_map = False

    #takes cell groups and creates dictionay 'label_map' that has attached color and marker
    def gene_color_map(self, args):

        if self.gene_list_filepath:
            gene_df1 = pd.read_csv(open(os.path.join(os.path.dirname(args.filepath), self.gene_list_filepath),'rU'), sep=None, engine='python')
            if self.exclude_list != []:
                gene_df = gene_df1.ix[~gene_df1['GeneID'].isin(self.exclude_list)]
            else:
                gene_df = gene_df1.copy()
            gene_list_1 = [g for g in list(set(gene_df['GeneID'].tolist())) if g in self.gene_list]
            if len(gene_df['GeneID']) == len(gene_df['GroupID']):
                self.gene_label_map ={}
                group_pos = 0
                group_seen = ['xyz' for i in range(len(set(gene_df['GroupID'].tolist())))]
                genes_seen = []
                for gene, group in zip(gene_df['GeneID'].tolist(), gene_df['GroupID'].tolist()):
                    #if no GroupIDs are provided replace withe empty strings
                    try:
                        if math.isnan(float(group)):
                            group = ' '
                    except ValueError:
                        pass
                    if gene not in genes_seen:
                        if str(group) in group_seen:
                            pos = group_seen.index(str(group))
                        else:
                            group_seen[group_pos] = str(group)
                            pos = group_pos
                            group_pos += 1
                        self.gene_label_map[gene] = (self.color_dict_genes[str(group)][0],self.color_dict_genes[str(group)][1],str(group))
                        genes_seen.append(gene)
                non_group_genes = [g for g in gene_list_1 if g not in genes_seen]
                if non_group_genes != []:
                    for cell in non_group_genes:
                        self.gene_label_map[gene] = (self.color_list[group_pos+1],self.markers[group_pos+1],'No_ID')
            else:
                self.gene_label_map = False
        else:
            self.gene_label_map = False



#!/usr/bin/env python

from __future__ import absolute_import, division, print_function
import matplotlib
matplotlib.use('TkAgg')
import numpy as np
import pandas as pd
import os
import sys
import collections

'''Cluster main function looks for either -gui flag and takes tkinter flags or
expects argparse arguments.Minimum input is a comma,tab, or space delimted
file of cells or experiments as columns and genes as the index.
'''

def main():
    #check for -gui flag
    try:
        from .scicast_argparse import check_gui_parser
    except SystemError:
        from scicast_argparse import check_gui_parser
    try:
        gui = check_gui_parser()
        run_gui = gui.gui_true
    except:
        run_gui = False
    if run_gui:
        try:
            from .tkinter_scicast import Window
        except SystemError:
            from tkinter_scicast import Window
        scicast_window = Window()
        scicast_window.mainloop()
        try:
            from .sci_load import Sci_load
        except:
            from sci_load import Sci_load
        scil = Sci_load()
        try:
            args = scil.load_options(all_options_dict = scicast_window.all_dict)
        except AttributeError:
            sys.exit('Please provide (at a minimum) a valid path to a file and click Run scicast.')

    #if -gui flag isn't found check for all other flags
    else:
        try:
            from .scicast_argparse import get_parser
        except SystemError:
            from scicast_argparse import get_parser
        args = get_parser()

    #try to make file in the directory of the file provided
    try:
        if args.base_name:
            new_file = os.path.join(os.path.dirname(args.filepath),args.base_name+'_scicast_analysis')
        #if no base name is provided just make the file as 'scicast_analysis'
        else:
            new_file = os.path.join(os.path.dirname(args.filepath),'scicast_analysis')
    except AttributeError:
        sys.exit('Please provide a valid path to a file.')
    if args.verbose:
        print('Making new folder for results of SCICAST clustering: '+new_file)
    try:
        os.mkdir(new_file)
    except OSError:
        if args.verbose:
            print(new_file+' already exists. Files will be overwritten.')

    #check for gene list file if full path is provided and in the same directory as the main file
    if args.gene_list_filename:
        if os.path.isfile(args.gene_list_filename):
            gene_file = args.gene_list_filename
        elif os.path.isfile(os.path.join(os.path.dirname(args.filepath),args.gene_list_filename)):
            gene_file = os.path.join(os.path.dirname(args.filepath),args.gene_list_filename)
        else:
            sys.exit('Error: Cannot find gene list file. Please place the gene list file in the same directory or provide a full path.')
    else:
        gene_file = False

    #check for cell list file if full path is provided and in the same directory as the main file
    if args.cell_list_filename:
        if os.path.isfile(args.cell_list_filename):
            cell_file = args.cell_list_filename
        elif os.path.isfile(os.path.join(os.path.dirname(args.filepath),args.cell_list_filename)):
            cell_file = os.path.join(os.path.dirname(args.filepath),args.cell_list_filename)
        else:
            sys.exit('Error: Cannot find cell list file. Please place the gene list file in the same directory or provide a full path.')
    else:
        cell_file=False

    #make main file in to pandas datframe
    try:
        by_cell = pd.read_csv(args.filepath, sep="\t", index_col=0)
    except OSError:
        sys.exit('Please provide a valid path to a file.')

    #check for duplicate gene names and if they exist drop them from the dataframe
    dup_gene_list = [item for item, count in collections.Counter(by_cell.index).items() if count > 1]
    if len(dup_gene_list) >0:
        by_cell.drop(dup_gene_list, inplace=True)
    #create list of genes
    gene_list_inital = by_cell.index.tolist()
    #create cell list
    cell_list = [x for x in list(by_cell.columns.values)]
    df_by_cell1 = pd.DataFrame(by_cell, columns=cell_list, index=gene_list_inital)

    #process matrix data into matrix class object matrix_data
    try:
        from .matrix_filter import Matrix_filter
    except SystemError:
        from matrix_filter import Matrix_filter
    matrix_data = Matrix_filter(cell_df=df_by_cell1 , args=args, cell_list_filepath=cell_file, gene_list_filepath=gene_file)



    #if stability iteration number is provided run heatmap clustering stability function
    stability_iteration_num = int(args.test_clust_stability)
    if  stability_iteration_num != 0:
        try:
            from .stability_test import clust_stability
        except SystemError:
            from stability_test import clust_stability
        clust_stability(args, matrix_data, stability_iteration_num)


    #parse kmeans cluster range if seperate by a dash or a comma
    if ',' not in args.kmeans_cluster_range and '-' not in args.kmeans_cluster_range and args.kmeans_cluster_range == '0':
        kmeans_range = []
    elif ',' in args.kmeans_cluster_range:
        kmeans_range = [int(k) for k in args.kmeans_cluster_range.split(',')]
    elif '-' in args.kmeans_cluster_range:
        kmeans_range = [int(k) for k in args.kmeans_cluster_range.split('-')]
    elif len(args.kmeans_cluster_range) == 1:
        kmeans_range = [int(args.kmeans_cluster_range), int(args.kmeans_cluster_range)]

    #run heatmaps and PCA only if no_heatmaps flag is not provided
    if not args.no_heatmaps:
        try:
            from .dim_reduction import plot_kmeans,plot_SVD,plot_TSNE,return_top_pca_gene
            from .heatmaps import clust_heatmap,make_tree_json,make_subclusters
        except SystemError:
            from dim_reduction import plot_kmeans,plot_SVD,plot_TSNE,return_top_pca_gene
            from heatmaps import clust_heatmap,make_tree_json,make_subclusters
        plot_SVD(args, matrix_data, title='all_cells')
        plot_TSNE(args, matrix_data, title='all_cells')

        #if kmeans range is provided run kmeans cluster detection
        if kmeans_range:
            plot_kmeans(args, matrix_data, kmeans_range=kmeans_range, title='all_cells')
        #
        if isinstance(matrix_data.log2_df_cell_gene_restricted, pd.DataFrame):
            top_pca_gene_df, top_pca = return_top_pca_gene(args, matrix_data.log2_df_cell_gene_restricted)
        else:
            top_pca_gene_df, top_pca = return_top_pca_gene(args, matrix_data.log2_df_cell)

        cell_linkage, plotted_df_by_gene, col_order = clust_heatmap(args, matrix_data, matrix_subset=top_pca_gene_df)
        cc = make_tree_json(cell_linkage, plotted_df_by_gene, new_file)
        make_subclusters(args, cc, matrix_data)

    if not args.no_corr:
        try:
            from .correlation import corr_plot
            from .dim_reduction import return_top_pca_gene
        except SystemError:
            from correlation import corr_plot
            from dim_reduction import return_top_pca_gene
        top_pca_gene_df, top_pca = return_top_pca_gene(args, matrix_data.log2_df_cell)

        top_genes_search = top_pca[0:50]
        corr_plot(top_genes_search, top_pca_gene_df, args, matrix_data, title = 'All_cells')

    if args.qgraph_plot == 'both':
        try:
            from .R_qgraph import run_qgraph
        except SystemError:
            from R_qgraph import run_qgraph
        run_qgraph(args, matrix_data, gene_or_cell='gene')
        run_qgraph(args, matrix_data, gene_or_cell='cell')
    elif args.qgraph_plot == 'gene':
        try:
            from .R_qgraph import run_qgraph
        except SystemError:
            from R_qgraph import run_qgraph
        run_qgraph(args, matrix_data, gene_or_cell='gene')
    elif args.qgraph_plot == 'cell':
        try:
            from .R_qgraph import run_qgraph
        except SystemError:
            from R_qgraph import run_qgraph
        run_qgraph(args, matrix_data, gene_or_cell='cell')
    if args.all_sig:
        try:
            from .heatmaps import find_twobytwo
            from .heatmaps import make_tree_json, clust_heatmap
        except:
            from heatmaps import find_twobytwo
            from heatmaps import make_tree_json, clust_heatmap
        cell_linkage, plotted_df_by_gene, col_order = clust_heatmap(args, matrix_data, plot=False)
        cc = make_tree_json(cell_linkage, plotted_df_by_gene, new_file)
        find_twobytwo(cc, args, matrix_data)
    if args.group_sig_test and args.cell_list_filename:
        try:
            from .significance_testing import multi_group_sig
        except SystemError:
            from significance_testing import multi_group_sig
        multi_group_sig(args, matrix_data)


if __name__ == "__main__":
    main()



from .cluster import *
from .scicast_argparse import *
from .sci_load import *
from .tkinter_scicast import *
from .matrix_filter import *
from .dim_reduction import *
from .heatmaps import *
from .correlation import *
from .significance_testing import *
from .R_qgraph import run_qgraph
from .stability_test import *
import matplotlib
matplotlib.use('TkAgg')

set()
__all__ = ["scicast_argparse", "sci_load", "tkinter_scicast", "matrix_filter", "dim_reduction", "heatmaps", "correlation", "significance_testing", "R_qgraph", "stability_test"]
__version__ = "0.8.01"



#! /usr/bin/env python
#
# Copyright (C) 2016 Ian Driver <ian.driver@ucsf.edu>
import os


DESCRIPTION = "scicast: Single Cell Iterative Clustering and Statistical Testing. A package for interrogating single cell sequencing data."
LONG_DESCRIPTION = """\
scicast is a python utility that automates many of the repetitive steps of analyzing single cell sequencing data.

-k-means clustering to identify clusters

-Clustering and subclustering of data to identify 'stable' sets of cells.

-Statistical testing to identify top genes that indentify stable cluster.

-Correlation search and analysis to identify gene networks driving cluster identity.

-Outputs both plots for visualization (PCA and heatmap) cell and gene lists that can be used to refine analysis.
"""

DISTNAME = 'scicast'
MAINTAINER = 'Ian Driver'
MAINTAINER_EMAIL = 'ian.driver@ucsf.edu'
URL = 'https://github.com/iandriver/scicast'
LICENSE = 'MIT'
DOWNLOAD_URL = 'https://github.com/iandriver/scicast'
VERSION = '0.8.01'

try:
    from setuptools import setup
    _has_setuptools = True
except ImportError:
    from distutils.core import setup

def check_dependencies():
    install_requires = []

    # Just make sure dependencies exist, I haven't rigorously
    # tested what the minimal versions that will work are
    # (help on that would be awesome)
    try:
        import numpy
    except ImportError:
        install_requires.append('numpy')
    try:
        import scipy
    except ImportError:
        install_requires.append('scipy')
    try:
        import matplotlib
    except ImportError:
        install_requires.append('matplotlib')
    try:
        import pandas
    except ImportError:
        install_requires.append('pandas>=0.19.0')
    try:
        import seaborn
    except ImportError:
        install_requires.append('seaborn>=0.7.1')
    try:
        import rpy2
    except ImportError:
        install_requires.append('rpy2')
    try:
        import fastcluster
    except ImportError:
        install_requires.append('fastcluster')

    return install_requires

if __name__ == "__main__":

    install_requires = check_dependencies()

    setup(name=DISTNAME,
        author=MAINTAINER,
        author_email=MAINTAINER_EMAIL,
        maintainer=MAINTAINER,
        maintainer_email=MAINTAINER_EMAIL,
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        license=LICENSE,
        url=URL,
        version=VERSION,
        py_modules=['scicast.cluster', 'scicast.matrix_filter'],
        entry_points={
          'console_scripts': ['scicast = scicast.cluster:main']
        },
        download_url=DOWNLOAD_URL,
        install_requires=install_requires,
        packages=['scicast'],
        keywords='single-cell single cell RNA-seq sequencing clustering PCA k-means',
        classifiers=[
                     'Intended Audience :: Science/Research',
                     'Programming Language :: Python :: 2.7',
                     'Programming Language :: Python :: 3.3',
                     'Programming Language :: Python :: 3.5',
                     'License :: OSI Approved :: MIT License',
                     'Topic :: Scientific/Engineering :: Visualization',
                     'Topic :: Scientific/Engineering :: Bio-Informatics',
                     'Operating System :: POSIX',
                     'Operating System :: Unix',
                     'Operating System :: MacOS'],
          )




import pandas as pd
import os
import matplotlib
matplotlib.use('TkAgg')
import matplotlib.pyplot as plt
import scipy
import seaborn as sns
import itertools
import matplotlib.patches as patches
import warnings


def group_gene_expression(args, matrix_data, threshold_of_expression = 1, from_kmeans='', alt_color_dict = {}, kmeans_groups_file = ''):
    path_filename = matrix_data.new_filepath
    full_by_cell_df = matrix_data.data_by_cell

    if alt_color_dict:
        color_dict_cell = alt_color_dict
        cell_groups_df = pd.read_csv(open(kmeans_groups_file,'rU'), sep=None, engine='python')
        group_name_list = list(set(cell_groups_df['GroupID']))

    else:
        color_dict_cell = matrix_data.color_dict_cells
        cell_groups_df = pd.read_csv(open(matrix_data.cell_list_filepath,'rU'), sep=None, engine='python')
        group_name_list = list(set(cell_groups_df['GroupID']))


    group_pairs = list(set(itertools.permutations(group_name_list,2)))
    gene_list = full_by_cell_df.index.tolist()
    cell_group_ident_0 = zip(cell_groups_df['SampleID'],cell_groups_df['GroupID'])
    cell_group_ident= [c for c in cell_group_ident_0]
    barplot_dict = {}
    by_gene_df = full_by_cell_df.transpose()
    best_gene_list = []
    best_gene_groups = []
    best_vs_list =[]
    best_pvalue_list = []
    for name in group_name_list:
        barplot_dict[name] = {'# genes expressed':[], 'pvalues':[], 'Vs':[]}
    for gp in group_pairs:
        index_list = []
        sig_gene_list = []
        gp_vs_all_seen = []
        sig_vs_all_gene_list = []
        cell_list1 = [c[0] for c in cell_group_ident if c[1] == gp[0]]
        cell_list2 = [c[0] for c in cell_group_ident if c[1] == gp[1]]
        cell1_present = [c for c in set(cell_list1) if c in set(full_by_cell_df.columns.tolist())]
        cell2_present = [c for c in set(cell_list2) if c in set(full_by_cell_df.columns.tolist())]
        all_other_cells =  [c for c in set(full_by_cell_df.columns.tolist()) if c not in set(cell_list1)]
        if cell1_present != [] and cell2_present != []:
            df_by_cell_1 = full_by_cell_df.loc[:,cell1_present]
            df_by_cell_2 = full_by_cell_df.loc[:,cell2_present]
            df_by_cell_other = full_by_cell_df.loc[:,all_other_cells]
            df_by_gene_1 = df_by_cell_1.transpose()
            df_by_gene_2 = df_by_cell_2.transpose()
            df_by_gene_other = df_by_cell_other.transpose()
            for g in gene_list:
                g_pvalue = scipy.stats.f_oneway(df_by_gene_1[g], df_by_gene_2[g])




'''Compares each defined cell group to each other group and returns all genes with p-value and adjusted p-value.
Also creates a best_gene_list with the top genes between each group, by adjusted p-value.
Also creates a barplot of top significant genes between groups (can be unique or not).
'''
def multi_group_sig(args, matrix_data, sig_to_plot = 20, from_kmeans='', alt_color_dict = {}, kmeans_groups_file = ''):
    if not args.verbose:
        warnings.filterwarnings("ignore", category=RuntimeWarning)
    path_filename = matrix_data.new_filepath
    #create seperate file for all of the significance files
    if from_kmeans == '':
        multi_sig_filename = os.path.join(path_filename,'user_defined_group_pairwise_significance_files')
    else:
        multi_sig_filename = os.path.join(path_filename,from_kmeans+'_group_pairwise_significance_files')
    try:
        os.mkdir(multi_sig_filename)
    except OSError:
        if args.verbose:
            print(multi_sig_filename+' already exists. Files will be overwritten.')

    full_by_cell_df = matrix_data.log2_df_cell

    if alt_color_dict:
        color_dict_cell = alt_color_dict
        cell_groups_df = pd.read_csv(open(kmeans_groups_file,'rU'), sep=None, engine='python')
        primary_group_name = 'GroupID'
        group_name_list = list(set(cell_groups_df[primary_group_name]))

    else:
        color_dict_cell = matrix_data.color_dict_cells
        cell_groups_df = pd.read_csv(open(matrix_data.cell_list_filepath,'rU'), sep=None, engine='python')
        group_name_list = matrix_data.cell_group_names
        primary_group_name = group_name_list[0]
        group_name_list = list(set(cell_groups_df[primary_group_name]))
    plot_pvalue = False
    from rpy2.robjects.packages import importr
    from rpy2.robjects.vectors import FloatVector
    stats = importr('stats')

    group_pairs = list(set(itertools.permutations(group_name_list,2)))
    gene_list = full_by_cell_df.index.tolist()
    cell_group_ident_0 = zip(cell_groups_df['SampleID'],cell_groups_df[primary_group_name])
    cell_group_ident= [c for c in cell_group_ident_0]
    barplot_dict = {}
    by_gene_df = full_by_cell_df.transpose()
    best_gene_list = []
    best_gene_groups = []
    best_vs_list =[]
    best_pvalue_list = []
    for name in group_name_list:
        barplot_dict[name] = {'genes':[], 'pvalues':[], 'fold_change':[], 'Vs':[]}
    for gp in group_pairs:
        index_list = []
        sig_gene_list = []
        gp_vs_all_seen = []
        sig_vs_all_gene_list = []
        cell_list1 = [c[0] for c in cell_group_ident if c[1] == gp[0]]
        cell_list2 = [c[0] for c in cell_group_ident if c[1] == gp[1]]
        cell1_present = [c for c in set(cell_list1) if c in set(full_by_cell_df.columns.tolist())]
        cell2_present = [c for c in set(cell_list2) if c in set(full_by_cell_df.columns.tolist())]
        all_other_cells =  [c for c in set(full_by_cell_df.columns.tolist()) if c not in set(cell_list1)]
        if cell1_present != [] and cell2_present != []:
            df_by_cell_1 = full_by_cell_df.loc[:,cell1_present]
            df_by_cell_2 = full_by_cell_df.loc[:,cell2_present]
            df_by_cell_other = full_by_cell_df.loc[:,all_other_cells]
            df_by_gene_1 = df_by_cell_1.transpose()
            df_by_gene_2 = df_by_cell_2.transpose()
            df_by_gene_other = df_by_cell_other.transpose()
            for g in gene_list:
                g_pvalue = scipy.stats.f_oneway(df_by_gene_1[g], df_by_gene_2[g])
                if gp[0] not in gp_vs_all_seen:
                    g_pvalue_all = scipy.stats.f_oneway(df_by_gene_1[g], df_by_gene_other[g])
                if g_pvalue[0] > 0 and g_pvalue[1] <= 1:
                    if g not in [s[0] for s in sig_gene_list]:
                        sig_gene_list.append([g, g_pvalue[1]])
                        if gp[0] not in gp_vs_all_seen:
                            sig_vs_all_gene_list.append([g, g_pvalue_all[1]])

            sig_gene_list.sort(key=lambda tup: tup[1])
            if gp[0] not in gp_vs_all_seen:
                sig_vs_all_gene_list.sort(key=lambda tup: tup[1])
                pvalues_all = [p[1] for p in sig_vs_all_gene_list]

                p_adjust_all = stats.p_adjust(FloatVector(pvalues_all), method = 'BH')
                gene_index_all = [ge[0] for ge in sig_vs_all_gene_list]
                mean_log2_exp_list_all = []
                sig_1_2_list_all = []
                mean1_list_all = []
                mean2_list_all = []
                for sig_gene in gene_index_all:
                    sig_gene_df_all = by_gene_df.loc[:,sig_gene]
                    mean_log2_exp_list_all.append(sig_gene_df_all.mean())
                    cell1_all = by_gene_df.loc[cell1_present,sig_gene]
                    mean_cell1_all = cell1_all.mean()
                    mean1_list_all.append(mean_cell1_all)
                    cell_other = by_gene_df.loc[all_other_cells,sig_gene]
                    mean_cell_other = cell_other.mean()
                    mean2_list_all.append(mean_cell_other)
                    ratio_1_other = (mean_cell1_all+1)/(mean_cell_other+1)
                    sig_1_2_list_all.append(ratio_1_other)
                sig_df_vs_other = pd.DataFrame({'pvalues':pvalues_all,'adjusted_p_values':p_adjust_all,'mean_all':mean_log2_exp_list_all, 'mean_'+gp[0]:mean1_list_all, 'mean_all_other':mean2_list_all, 'ratio '+gp[0]+' to everything':sig_1_2_list_all}, index=gene_index_all)
                sig_df_vs_other.to_csv(os.path.join(multi_sig_filename,'sig_'+gp[0]+'_VS_all_other_pvalues.txt'), sep = '\t')
                gp_vs_all_seen.append(gp[0])
            pvalues = [p[1] for p in sig_gene_list]
            p_adjust = stats.p_adjust(FloatVector(pvalues), method = 'BH')
            gene_index = [ge[0] for ge in sig_gene_list]

            mean_log2_exp_list = []
            sig_1_2_list = []
            mean1_list = []
            mean2_list = []
            for sig_gene in gene_index:
                sig_gene_df = by_gene_df.loc[:,sig_gene]
                mean_log2_exp_list.append(sig_gene_df.mean())

                cell1 = by_gene_df.loc[cell1_present,sig_gene]
                mean_cell1 = cell1.mean()
                mean1_list.append(mean_cell1)
                cell2 = by_gene_df.loc[cell2_present,sig_gene]
                mean_cell2 = cell2.mean()
                mean2_list.append(mean_cell2)
                ratio_1_2 = (mean_cell1+1)/(mean_cell2+1)
                sig_1_2_list.append(ratio_1_2)
            sig_df = pd.DataFrame({'pvalues':pvalues,'adjusted_p_values':p_adjust,'mean_all':mean_log2_exp_list,'mean_'+gp[0]:mean1_list, 'mean_'+gp[1]:mean2_list, 'ratio '+gp[0]+' to '+gp[1]:sig_1_2_list}, index=gene_index)
            cell_names_df = pd.DataFrame({gp[0]+'_cells':pd.Series(cell1_present, index=range(len(cell1_present))), gp[1]+'_cells2':pd.Series(cell2_present, index=range(len(cell2_present)))})
            sig_df.to_csv(os.path.join(multi_sig_filename,'sig_'+gp[0]+'_VS_'+gp[1]+'_pvalues.txt'), sep = '\t')
            cell_names_df.to_csv(os.path.join(multi_sig_filename,'sig_'+gp[0]+'_VS_'+gp[1]+'_cells.txt'), sep = '\t')


            top_fc_df1 = sig_df.loc[(sig_df['ratio '+gp[0]+' to '+gp[1]]>1.3)]

            top_fc_df = top_fc_df1.sort_values(by='adjusted_p_values',axis=0, ascending=True)
            genes = top_fc_df.index.tolist()
            pvalues = top_fc_df['adjusted_p_values'].tolist()
            fc = top_fc_df.loc[:,'ratio '+gp[0]+' to '+gp[1]].tolist()
            z = zip(genes,pvalues,fc)
            z_all = [s for s in z]
            if args.sig_unique:
                top_t = [g for g in z_all if g[0] not in barplot_dict[gp[0]]['genes']]
            else:
                top_t = [g for g in z_all if g[0]]
            if matrix_data.exclude_list != None:
                top_t2 = [g for g in top_t if g[0] not in barplot_dict[gp[0]]['genes'] and g[0] not in matrix_data.exclude_list]
            else:
                top_t2 = top_t
            top = [list(t) for t in zip(*top_t2)]
            try:
                sig_to_plot = min(len(top[0]),sig_to_plot)
            except IndexError:
                sig_to_plot = 0
            if sig_to_plot != 0:
                barplot_dict[gp[0]]['genes']= barplot_dict[gp[0]]['genes']+[str(gene.strip(' ')) for gene in top[0][0:sig_to_plot]]
                barplot_dict[gp[0]]['pvalues']= barplot_dict[gp[0]]['pvalues']+top[1][0:sig_to_plot]
                barplot_dict[gp[0]]['fold_change']= barplot_dict[gp[0]]['fold_change']+top[2][0:sig_to_plot]
                barplot_dict[gp[0]]['Vs']= barplot_dict[gp[0]]['Vs']+ ['significance vs '+gp[1] for x in range(0,len(top[0][0:sig_to_plot]))]
                best_gene_list = best_gene_list+top[0][0:sig_to_plot]
                best_gene_groups = best_gene_groups+[gp[0] for x in range(0,len(top[0][0:sig_to_plot]))]
                best_vs_list = best_vs_list + [gp[1] for x in range(0,len(top[0][0:sig_to_plot]))]
                best_pvalue_list = best_pvalue_list + top[1][0:sig_to_plot]
        else:
            if cell1_present == []:
                if args.verbose:
                    print(gp[1], 'not present in cell matrix')
            else:
                if args.verbose:
                    print(gp[0], 'not present in cell matrix')
    fig, axs = plt.subplots(1, len(group_name_list), figsize=(23+len(group_name_list),13), sharex=False, sharey=False)
    axs = axs.ravel()
    color_map = {}

    #plot top significant genes for each group compared to all other groups
    for i, name in enumerate(group_name_list):
        to_plot= barplot_dict[name]
        for v in set(to_plot['Vs']):
            color_map[v] = color_dict_cell[v.split("significance vs ")[-1]][0]
        if color_map != {}:
            g = sns.barplot(x='pvalues', y='genes', hue='Vs', data=to_plot, ax = axs[i], palette=color_map)
            axs[i].set_xscale("log", nonposx='clip')
            bar_list = []
            if plot_pvalue:
                for p in axs[i].patches:
                    height = p.get_height()
                    width = p.get_width()
                    bar_list.append(width)
                max_bar = max(bar_list)
                for p in axs[i].patches:
                    height = p.get_height()
                    width = p.get_width()
                    axs[i].text(max_bar*50,p.get_y()+(height), "{:.2e}".format(width))
            rect = axs[i].patch
            rect.set_facecolor('white')
            #sns.despine(left=True, bottom=True, top=True)
            axs[i].invert_xaxis()
            axs[i].xaxis.set_ticks_position('none')
            axs[i].yaxis.tick_right()
            axs[i].set_title(name)
            axs[i].legend(loc='upper left', bbox_to_anchor=(0.01, 1.11+(0.01*len(group_name_list))), ncol=1, prop={'size':15})
            axs[i].set_xlabel('adjusted p-value')
            for xmaj in axs[i].xaxis.get_majorticklocs():
                axs[i].axvline(x=xmaj,ls='--', lw = 0.5, color='grey', alpha=0.3)
            axs[i].xaxis.grid(True, which="major", linestyle='-')
            plt.subplots_adjust(left=.08, wspace=.3)
    if from_kmeans == '':
        plt.savefig(os.path.join(path_filename,'differential_genes_foldchanges.pdf'), bbox_inches='tight')
        best_gene_df = pd.DataFrame({'GeneID':best_gene_list, 'GroupID':best_gene_groups, 'Vs':best_vs_list, 'adjusted_pvalue':best_pvalue_list})
        best_gene_df.to_csv(os.path.join(path_filename,'Best_Gene_list.txt'), sep = '\t')
    else:
        plt.savefig(os.path.join(path_filename,from_kmeans+'_differential_genes_foldchanges.pdf'), bbox_inches='tight')
        best_gene_df = pd.DataFrame({'GeneID':best_gene_list, 'GroupID':best_gene_groups, 'Vs':best_vs_list, 'adjusted_pvalue':best_pvalue_list})
        best_gene_df.to_csv(os.path.join(path_filename,from_kmeans+'_Best_Gene_list.txt'), sep = '\t')




class Sci_load(object):
    def __init__(self):
        pass


    def load_options(self, all_options_dict):

        self.filepath = all_options_dict['filepath']
        self.base_name = all_options_dict['output_name']
        self.method = all_options_dict['method']
        self.metric = all_options_dict['metric']
        self.gene_number = all_options_dict['gene_number']
        self.depth_of_clustering = all_options_dict['depth']
        self.cell_list_filename = all_options_dict['cell_file']
        self.gene_list_filename = all_options_dict['gene_file']
        self.z_direction = all_options_dict['zdir']
        self.qgraph_plot = all_options_dict['qgraph']
        self.exclude_genes = all_options_dict['exclude_genes']
        self.kmeans_cluster_range = all_options_dict['kmeans_cluster_range']
        self.color_cells = all_options_dict['color_cells']
        self.color_genes = all_options_dict['color_genes']
        self.genes_corr = all_options_dict['genes_corr']
        self.test_clust_stability = all_options_dict['test_clust_stability']
        self.annotate_gene_subset = all_options_dict['annotate_gene_subset']
        self.no_heatmaps = all_options_dict["Don't Run Heatmaps"]
        self.no_corr = all_options_dict["Don't Run Correlation"]
        self.verbose = all_options_dict["Verbose"]
        self.group_sig_test = all_options_dict["Test Significance by Groups (User Defined)"]
        self.all_sig = all_options_dict["Test Significance by Unbiased Clusters"]
        self.limit_cells= all_options_dict["Exclude Cells Not in User Cell Groups"]
        self.add_ellipse= all_options_dict["Add Ellipse"]
        self.annotate_cell_pca= all_options_dict["Add Cell Names to PCA"]
        self.sig_unique= all_options_dict["Display Only Unique Signifcant Genes"]
        self.kmeans_sig_test = all_options_dict["Run Significance Test for kmeans clusters"]
        self.already_log2 = all_options_dict["Input Matrix is already log2"]
        return(self)
